@@ -15,20 +15,14 @@
 ############
 
 import os
-import tempfile
-from shutil import copy, copytree
-from urllib.parse import urlparse
+from urlparse import urlparse
 
-from cloudify_cli import utils
-from cloudify_cli.exceptions import CloudifyCliError
-from cloudify_cli.constants import DEFAULT_BLUEPRINT_PATH
+from . import utils
+from .exceptions import CloudifyCliError
+from .constants import DEFAULT_BLUEPRINT_PATH
 
 
-ICON_FILENAME = 'icon.png'
-
-
-def get(source, blueprint_filename=DEFAULT_BLUEPRINT_PATH, icon_path=None,
-        download=False):
+def get(source, blueprint_filename=DEFAULT_BLUEPRINT_PATH, download=False):
     """Get a source and return a directory containing the blueprint
 
     The behavior based on then source argument content is:
@@ -48,36 +42,30 @@ def get(source, blueprint_filename=DEFAULT_BLUEPRINT_PATH, icon_path=None,
     :type source: str
     :param blueprint_filename: Path to blueprint (if source is an archive file)
     :type blueprint_filename: str
-    :param icon_path: Path to blueprint's icon file
-    :type icon_path: str
     :param download: Download blueprint file if source is URL/github repo
     :type download: bool
     :return: Path to file (if archive/blueprint file passsed) or url
     :rtype: str
 
     """
-    # Cope with windows (where paths always have a scheme)
-    if urlparse(source).scheme and not os.path.exists(source):
+    if urlparse(source).scheme:
         if download:
             downloaded_file = utils.download_file(source)
             return _get_blueprint_file_from_archive(
-                downloaded_file, blueprint_filename, icon_path)
+                downloaded_file, blueprint_filename)
         return source
     elif os.path.isfile(source):
         if utils.is_archive(source):
-            return _get_blueprint_file_from_archive(
-                source, blueprint_filename, icon_path)
-        elif icon_path:
-            return _get_blueprint_file_with_icon(source, icon_path)
+            return _get_blueprint_file_from_archive(source, blueprint_filename)
         else:
             # Maybe check if yaml. If not, verified by dsl parser
             return source
     elif len(source.split('/')) == 2:
         url = _map_to_github_url(source)
         if download:
-            downloaded_file = utils.download_file(url)
+            downloaded_file = utils.download_file(source)
             return _get_blueprint_file_from_archive(
-                downloaded_file, blueprint_filename, icon_path)
+                downloaded_file, blueprint_filename)
         return url
     else:
         raise CloudifyCliError(
@@ -85,15 +73,13 @@ def get(source, blueprint_filename=DEFAULT_BLUEPRINT_PATH, icon_path=None,
             'or a GitHub `organization/repository[:tag/branch]`')
 
 
-def _get_blueprint_file_from_archive(archive, blueprint_filename, icon_path):
+def _get_blueprint_file_from_archive(archive, blueprint_filename):
     """Extract archive to temporary location and get path to blueprint file.
 
     :param archive: Path to archive file
     :type archive: str
     :param blueprint_filename: Path to blueprint file relative to archive
     :type blueprint_filename: str
-    :param icon_path: Absolute path to blueprint's icon
-    :type icon_path: str
     :return: Absolute path to blueprint file
     :rtype: str
 
@@ -109,33 +95,9 @@ def _get_blueprint_file_from_archive(archive, blueprint_filename, icon_path):
             'Could not find `{0}`. Please provide the name of the main '
             'blueprint file by using the `-n/--blueprint-filename` flag'
             .format(blueprint_filename))
-    if icon_path:
-        icon_file = os.path.join(blueprint_directory, ICON_FILENAME)
-        copy(icon_path, icon_file)
-
     return blueprint_file
 
 
-def _get_blueprint_file_with_icon(blueprint_path, icon_path):
-    """Create a temporary directory with a blueprint file and its icon.
-
-    :param blueprint_path: Absolute path to the blueprint file
-    :type blueprint_path: str
-    :param icon_path: Absolute path to blueprint's icon
-    :type icon_path: str
-    :return: Absolute path to blueprint file
-    :rtype: str
-
-    """
-    source, blueprint_filename = os.path.split(blueprint_path)
-    source = source or os.curdir
-    blueprint_directory = os.path.join(tempfile.mkdtemp(),
-                                       blueprint_filename.rpartition('.')[0])
-    copytree(source, blueprint_directory)
-    copy(icon_path, os.path.join(blueprint_directory, ICON_FILENAME))
-    return os.path.join(blueprint_directory, blueprint_filename)
-
-
 def _map_to_github_url(source):
     """Returns a path to a downloaded github archive.
 
