@@ -25,18 +25,17 @@ except ImportError:
 
 from cloudify.logs import create_event_message_prefix
 
-from cloudify_cli import env, utils
-from cloudify_cli.cli import cfy
-from cloudify_cli.exceptions import CloudifyCliError
-from cloudify_cli.execution_events_fetcher import (
-    wait_for_execution,
-    WAIT_FOR_EXECUTION_SLEEP_INTERVAL)
-from cloudify_cli.table import print_data
+from ..cli import cfy
+from ..execution_events_fetcher import (wait_for_execution,
+                                        WAIT_FOR_EXECUTION_SLEEP_INTERVAL)
+from ..exceptions import CloudifyCliError
+from .. import env
+from ..table import print_data
 
 
 _NODE_INSTANCE_STATE_STARTED = 'started'
-AGENT_COLUMNS = ['id', 'ip', 'deployment', 'state', 'node', 'system',
-                 'version', 'install_method', 'tenant_name']
+AGENT_COLUMNS = ['id', 'ip', 'deployment', 'node', 'system', 'version',
+                 'install_method', 'tenant_name']
 
 MAX_TRACKER_THREADS = 20
 
@@ -53,16 +52,11 @@ def agents():
 @agents.command(name='list',
                 short_help='List installed agents [manager only]')
 @cfy.options.common_options
-@cfy.options.tenant_name(required=False,
-                         mutually_exclusive_with=['all_tenants'],
-                         resource_name_for_help='relevant deployment(s)')
 @cfy.options.agent_filters
 @cfy.options.all_tenants
 @cfy.pass_logger
 @cfy.pass_client()
-@cfy.options.extended_view
-def agents_list(agent_filters, tenant_name, client, logger, all_tenants):
-    utils.explicit_tenant_name_message(tenant_name, logger)
+def agents_list(agent_filters, client, logger, all_tenants):
     agent_filters['_all_tenants'] = all_tenants
     agent_list = client.agents.list(**agent_filters)
     logger.info('Listing agents...')
@@ -72,9 +66,8 @@ def agents_list(agent_filters, tenant_name, client, logger, all_tenants):
 @agents.command(name='install',
                 short_help='Install deployment agents [manager only]')
 @cfy.options.common_options
-@cfy.options.tenant_name(required=False,
-                         mutually_exclusive_with=['all_tenants'],
-                         resource_name_for_help='relevant deployment(s)')
+@cfy.options.tenant_name_for_list(
+    required=False, resource_name_for_help='relevant deployment(s)')
 @cfy.options.all_tenants
 @cfy.options.stop_old_agent
 @cfy.options.manager_ip
@@ -107,7 +100,6 @@ def install(agent_filters,
         params['manager_ip'] = manager_ip
         params['manager_certificate'] = manager_certificate
     params['install_agent_timeout'] = install_agent_timeout
-    utils.explicit_tenant_name_message(tenant_name, logger)
     get_deployments_and_run_workers(
         client, agent_filters, all_tenants,
         logger, 'install_new_agents', wait, params)
@@ -225,38 +217,30 @@ def get_filters_map(
         # state.
         # We skip this check if specific deployment ID's were requested.
         if not requested_deployment_ids:
-            for tenant_name in list(tenants_to_deployments):
+            for tenant_name in tenants_to_deployments.keys():
                 tenant_client = env.get_rest_client(tenant_name=tenant_name)
                 deps_to_execute = tenants_to_deployments[tenant_name]
-                offset = 0
-                while True:
-                    node_instances = tenant_client.node_instances.list(
-                        _include=['id', 'host_id', 'deployment_id', 'state'],
-                        _offset=offset,
-                    )
-                    # Find all unstarted Compute instances.
-                    unstarted_computes = [
-                        ni for ni in node_instances
-                        if ni.id == ni.host_id and
-                        ni.state != _NODE_INSTANCE_STATE_STARTED
-                    ]
 
-                    for unstarted_ni in unstarted_computes:
-                        logger.info("Node instance '%s' is not in '%s' state; "
-                                    "deployment '%s' will be skipped",
-                                    unstarted_ni.id,
-                                    _NODE_INSTANCE_STATE_STARTED,
-                                    unstarted_ni.deployment_id)
-                        deps_to_execute.pop(unstarted_ni.deployment_id, None)
+                node_instances = tenant_client.node_instances.list(
+                    deployment_id=deps_to_execute.keys(),
+                    _include=['id', 'host_id', 'deployment_id', 'state']
+                )
 
-                    if not deps_to_execute:
-                        del tenants_to_deployments[tenant_name]
+                # Find all unstarted Compute instances.
+                unstarted_computes = list(filter(
+                    lambda ni: ni.id == ni.host_id and ni.state !=
+                    _NODE_INSTANCE_STATE_STARTED,
+                    node_instances))
 
-                    size = node_instances.metadata.pagination.size
-                    total = node_instances.metadata.pagination.total
-                    if len(node_instances) < size or size == total:
-                        break
-                    offset += size
+                for unstarted_ni in unstarted_computes:
+                    logger.info("Node instance '%s' is not in '%s' state; "
+                                "deployment '%s' will be skipped",
+                                unstarted_ni.id, _NODE_INSTANCE_STATE_STARTED,
+                                unstarted_ni.deployment_id)
+                    deps_to_execute.pop(unstarted_ni.deployment_id, None)
+
+                if not deps_to_execute:
+                    del tenants_to_deployments[tenant_name]
 
     return tenants_to_deployments
 
@@ -271,6 +255,7 @@ def get_deployments_and_run_workers(
         parameters=None):
     tenants_to_deployments = get_filters_map(
         client, logger, agent_filters, all_tenants)
+
     if not tenants_to_deployments:
         raise CloudifyCliError("No eligible deployments found")
 
@@ -374,9 +359,8 @@ def get_deployments_and_run_workers(
                            ' (installed on remote hosts). [manager only]')
 @cfy.options.common_options
 @cfy.options.agent_filters
-@cfy.options.tenant_name(required=False,
-                         mutually_exclusive_with=['all_tenants'],
-                         resource_name_for_help='relevant deployment(s)')
+@cfy.options.tenant_name_for_list(
+    required=False, resource_name_for_help='relevant deployment(s)')
 @cfy.options.all_tenants
 @cfy.options.agents_wait
 @cfy.pass_logger
@@ -390,7 +374,6 @@ def validate(agent_filters,
     """Validates the connection between the Cloudify Manager and the
     live Cloudify Agents (installed on remote hosts).
     """
-    utils.explicit_tenant_name_message(tenant_name, logger)
     get_deployments_and_run_workers(
         client, agent_filters, all_tenants,
         logger, 'validate_agents', wait, None)
