@@ -21,23 +21,25 @@ from cloudify.cluster_status import CloudifyNodeType
 from cloudify_rest_client.exceptions import CloudifyClientError, \
     UserUnauthorizedError
 
-from cloudify_cli import env
-from cloudify_cli.cli import cfy
-from cloudify_cli.env import profile
-from cloudify_cli.exceptions import CloudifyCliError
-from cloudify_cli.logger import (
+from .. import env
+from ..cli import cfy
+from ..env import profile
+from ..exceptions import CloudifyCliError
+from ..table import print_data, print_details
+from ..logger import (
     output,
     get_logger,
     CloudifyJSONEncoder,
     get_global_json_output
 )
-from cloudify_cli.table import print_data, print_details
 
 # The list will be updated with the services on each manager
 MANAGER_COLUMNS = ['hostname', 'private_ip', 'public_ip', 'version', 'edition',
-                   'distribution', 'distro_release', 'last_seen', 'networks']
-BROKER_COLUMNS = ['name', 'port', 'networks', 'is_external', 'host']
-DB_COLUMNS = ['name', 'host', 'is_external']
+                   'distribution', 'distro_release', 'node_id', 'last_seen',
+                   'networks']
+BROKER_COLUMNS = ['name', 'node_id', 'port',
+                  'networks', 'is_external', 'host']
+DB_COLUMNS = ['name', 'node_id', 'host', 'is_external']
 
 
 def check_manager_exists(managers, hostname, must_exist=True):
@@ -113,10 +115,10 @@ def _all_in_one_manager(client):
             return False
         if len(manager_nodes) == 1:
             if len(broker_nodes) == len(db_nodes_items) == 1:
-                db_node_ip = db_nodes_items[0].get('host')
-                broker_node_ip = broker_nodes[0].get('host')
-                manager_node_ip = manager_nodes[0].get('private_ip')
-                if db_node_ip == broker_node_ip == manager_node_ip:
+                db_node_id = db_nodes_items[0].get('node_id')
+                broker_node_id = broker_nodes[0].get('node_id')
+                manager_node_id = manager_nodes[0].get('node_id')
+                if db_node_id == broker_node_id == manager_node_id:
                     return True
 
             get_logger().warning('It is highly recommended to have more '
@@ -139,7 +141,7 @@ def _all_in_one_manager(client):
 @cfy.options.common_options
 def cluster():
     """
-    Handle the Cloudify Manager cluster (Premium feature)
+    Handle the Cloudify Manager cluster
     """
     if not env.is_initialized():
         env.raise_uninitialized()
@@ -261,7 +263,7 @@ def _update_cluster_nodes(nodes, nodes_type, logger):
         _update_node(node, nodes_type, logger, stored_nodes_names)
     # filter out removed nodes
     env.profile.cluster[nodes_type] = [
-        node for node in env.profile.cluster.get(nodes_type, {})
+        node for node in env.profile.cluster[nodes_type]
         if _get_node_host(node) in received_nodes_names]
     env.profile.save()
 
@@ -269,10 +271,7 @@ def _update_cluster_nodes(nodes, nodes_type, logger):
 def _update_node(node, node_type, logger, stored_nodes_names):
     if _get_node_host(node) not in stored_nodes_names:
         if node_type == CloudifyNodeType.MANAGER:
-            if env.profile.name == "manager-local":
-                node_ip = node['private_ip']
-            else:
-                node_ip = node['public_ip'] or node['private_ip']
+            node_ip = node['public_ip'] or node['private_ip']
         else:
             node_ip = node['host']
         if logger:
@@ -327,7 +326,6 @@ def get_broker(client, logger, name):
 @pass_cluster_client()
 @cfy.pass_logger
 @cfy.options.common_options
-@cfy.options.extended_view
 def list_brokers(client, logger):
     """List brokers associated with the cluster."""
     _list_brokers(client, logger)
@@ -355,8 +353,10 @@ def _list_brokers(client, logger):
 @cfy.argument('address')
 @cfy.options.port
 @cfy.options.networks(required=False)
+@cfy.options.node_id
 @cfy.options.common_options
-def add_broker(client, logger, name, address, port=None, networks=None):
+def add_broker(client, logger, name, address, port=None, networks=None,
+               node_id=None):
     """Register a broker with the cluster.
 
     Note that this will not create the broker itself. The broker should have
@@ -365,7 +365,7 @@ def add_broker(client, logger, name, address, port=None, networks=None):
     check_broker_exists(client.manager.get_brokers().items, name,
                         must_exist=False)
 
-    client.manager.add_broker(name, address, port, networks)
+    client.manager.add_broker(name, address, port, networks, node_id)
 
     logger.info('Broker {0} was added successfully!'
                 .format(name))
@@ -432,23 +432,11 @@ def db_nodes():
         env.raise_uninitialized()
 
 
-@db_nodes.command(name='update',
-                  short_help="Make managers act upon changes to the DB nodes")
-@pass_cluster_client()
-@cfy.pass_logger
-@cfy.options.common_options
-@cfy.options.extended_view
-def update_db_nodes(client, logger):
-    db_nodes_list = client.manager.update_db_nodes()
-    print_data(DB_COLUMNS, db_nodes_list, 'HA Cluster db nodes')
-
-
 @db_nodes.command(name='list',
                   short_help="List the DB cluster's nodes")
 @pass_cluster_client()
 @cfy.pass_logger
 @cfy.options.common_options
-@cfy.options.extended_view
 def list_db_nodes(client, logger):
     db_nodes_list = client.manager.get_db_nodes()
     print_data(DB_COLUMNS, db_nodes_list, 'HA Cluster db nodes')
@@ -467,7 +455,6 @@ def managers():
 @pass_cluster_client()
 @cfy.pass_logger
 @cfy.options.common_options
-@cfy.options.extended_view
 def list_managers(client, logger):
     managers_list = client.manager.get_managers()
     print_data(MANAGER_COLUMNS, managers_list, 'HA Cluster manager nodes')
