@@ -33,7 +33,7 @@ from ..exceptions import CloudifyCliError
               '--args',
               multiple=True,
               help=helptexts.DEV_TASK_ARGS)
-@cfy.options.verbose()
+@cfy.options.common_options
 @cfy.assert_manager_active()
 def dev(tasks_file, task, args):
     """Run fabric tasks on the manager
