@@ -16,15 +16,24 @@
 
 
 import os
+import shutil
+import pkg_resources
 
-from cloudify_cli import blueprint, env, exceptions, local
-from cloudify_cli.cli import cfy
-from cloudify_cli.config import config
-from cloudify_cli.exceptions import CloudifyCliError
-from cloudify_cli.logger import DEFAULT_LOG_FILE, configure_loggers
+from jinja2.environment import Template
+
+import cloudify_cli
+from .. import env
+from .. import local
+from ..cli import cfy
+from .. import blueprint
+from .. import exceptions
+from ..config import config
+from ..logger import DEFAULT_LOG_FILE
+from ..logger import configure_loggers
+from ..exceptions import CloudifyCliError
 
 
-@cfy.command(name='init',)
+@cfy.command(name='init', short_help='Initialize a working env')
 @cfy.argument('blueprint-path', required=False)
 @cfy.options.blueprint_filename()
 @cfy.options.blueprint_id(required=False, validate=True)
@@ -59,7 +68,6 @@ def init(blueprint_path,
     ~/.cloudify/config.yaml. For more information refer to the docs
     at http://docs.getcloudify.org
     """
-
     profile_name = 'local'
 
     if blueprint_path:
@@ -84,8 +92,9 @@ def init(blueprint_path,
             processed_blueprint_path,
             blueprint_filename
         )
-        if local.blueprint_exists(blueprint_id):
-            local.remove(blueprint_id)
+
+        if os.path.isdir(local.storage_dir(blueprint_id)):
+            shutil.rmtree(local.storage_dir(blueprint_id))
 
         try:
             storage = local.get_storage()
@@ -139,11 +148,10 @@ def init_manager_profile(profile_name,
                          reset_context=False,
                          hard=False,
                          enable_colors=False,
-                         logger=None,
-                         profile=None):
+                         logger=None):
     logger.info('Initializing profile {0}...'.format(profile_name))
 
-    context_file_path = env.get_context_path(profile_name)
+    context_file_path = env.get_context_path(profile_name, suppress_error=True)
 
     if context_file_path and os.path.isfile(context_file_path):
         if reset_context:
@@ -156,9 +164,8 @@ def init_manager_profile(profile_name,
 
     _create_profiles_dir_and_config(hard, enable_colors)
 
-    if not profile:
-        profile = env.ProfileContext()
-        profile.manager_ip = profile_name
+    profile = env.ProfileContext()
+    profile.manager_ip = profile_name
     profile.save()
 
     logger.info('Initialization completed successfully')
@@ -166,7 +173,7 @@ def init_manager_profile(profile_name,
 
 def _create_profiles_dir_and_config(hard, enable_colors):
     if not os.path.isdir(env.PROFILES_DIR):
-        os.makedirs(env.PROFILES_DIR, mode=0o700)
+        os.makedirs(env.PROFILES_DIR)
     if not env.config_initialized_with_logging() or hard:
         set_config(enable_colors=enable_colors)
 
@@ -183,24 +190,16 @@ def _raise_initialized_error(profile_name):
 
 
 def set_config(enable_colors=False):
-    config_template = """---
-colors: {enable_colors}
+    cli_config = pkg_resources.resource_string(
+        cloudify_cli.__name__,
+        'config/config_template.yaml')
 
-logging:
-
-    # path to a file where cli logs will be saved.
-    filename: {log_path}
-
-    # configuring level per logger
-    loggers:
-
-        # main logger of the cli. provides basic descriptions for executed operations.
-        cloudify.cli.main: info
-
-        # rest client http logs, including headers.
-        cloudify.rest_client.http: info
-"""  # noqa
-    with open(config.CLOUDIFY_CONFIG_PATH, 'w') as f:
-        f.write(config_template.format(enable_colors=enable_colors,
-                                       log_path=DEFAULT_LOG_FILE))
+    enable_colors = str(enable_colors).lower()
+    template = Template(cli_config)
+    rendered = template.render(
+        log_path=DEFAULT_LOG_FILE,
+        enable_colors=enable_colors
+    )
+    with open(config.CLOUDIFY_CONFIG_PATH, 'a') as f:
+        f.write(rendered)
         f.write(os.linesep)
