@@ -16,10 +16,12 @@
 
 import time
 
-from cloudify_cli import exceptions
-from cloudify_cli.cli import cfy
-from cloudify_cli.logger import NO_VERBOSE, get_global_verbosity
-from cloudify_cli.table import print_data
+from .. import utils
+from ..cli import cfy
+from .. import exceptions
+from ..table import print_data
+from ..logger import NO_VERBOSE
+from ..logger import get_global_verbosity
 
 
 DEFAULT_TIMEOUT_INTERVAL = 5
@@ -41,7 +43,6 @@ def maintenance_mode():
                           '[manager only]')
 @cfy.options.common_options
 @cfy.pass_client()
-@cfy.options.extended_view
 def status(client):
     """Retrieve the current maintenance-mode status.
     """
@@ -53,7 +54,8 @@ def _print_maintenance_mode_status(client, logger):
     status_response = client.maintenance_mode.status()
 
     logger.info('\nMaintenance Mode Status:')
-    for param_name, param_value in status_response.items():
+    for param_name, param_value in utils.decode_dict(
+            status_response).iteritems():
         if param_value and param_name != 'remaining_executions':
             logger.info('\t{0}:\t{1}'.format(
                 param_name.title().replace("_", " "),
