@@ -16,18 +16,17 @@
 
 import json
 
-from cloudify_rest_client.exceptions import CloudifyClientError
+import click
 
-from cloudify_cli import utils
-from cloudify_cli.cli import cfy, helptexts
-from cloudify_cli.exceptions import CloudifyCliError
-from cloudify_cli.local import load_env
-from cloudify_cli.logger import get_global_json_output
-from cloudify_cli.table import print_data, print_details, print_single
-from cloudify_cli.utils import deep_update_dict, deep_subtract_dict
-from cloudify_cli.commands.summary import (
-    BASE_SUMMARY_FIELDS,
-    structure_summary_results)
+from .. import utils
+from ..cli import cfy
+from ..local import load_env
+from ..exceptions import CloudifyCliError
+from ..logger import get_global_json_output
+from ..utils import deep_update_dict, deep_subtract_dict
+from ..table import print_data, print_details, print_single
+from cloudify_rest_client.exceptions import CloudifyClientError
+from .summary import BASE_SUMMARY_FIELDS, structure_summary_results
 
 
 NODE_INSTANCE_COLUMNS = ['id', 'deployment_id', 'host_id', 'node_id', 'state',
@@ -56,22 +55,17 @@ def node_instances():
 @cfy.options.common_options
 @cfy.options.tenant_name(
     required=False, resource_name_for_help='node-instance')
-@cfy.options.evaluate_functions
 @cfy.pass_logger
 @cfy.pass_client()
-@cfy.options.extended_view
-def get(node_instance_id, evaluate_functions, logger, client, tenant_name):
+def get(node_instance_id, logger, client, tenant_name):
     """Retrieve information for a specific node-instance
 
     `NODE_INSTANCE_ID` is the id of the node-instance to get information on.
     """
     utils.explicit_tenant_name_message(tenant_name, logger)
-    logger.info('Retrieving node instance %s', node_instance_id)
+    logger.info('Retrieving node instance {0}'.format(node_instance_id))
     try:
-        node_instance = client.node_instances.get(
-            node_instance_id,
-            evaluate_functions=evaluate_functions,
-        )
+        node_instance = client.node_instances.get(node_instance_id)
     except CloudifyClientError as e:
         if e.status_code != 404:
             raise
@@ -98,7 +92,6 @@ def get(node_instance_id, evaluate_functions, logger, client, tenant_name):
 @cfy.options.common_options
 @cfy.pass_logger
 @cfy.pass_client()
-@cfy.options.extended_view
 def list(deployment_id,
          node_name,
          sort_by,
@@ -145,16 +138,11 @@ def list(deployment_id,
 
 @node_instances.command(name='summary',
                         short_help='Retrieve summary of node instance '
-                                   'details [manager only]',
-                        help=helptexts.SUMMARY_HELP.format(
-                            type='node-instances',
-                            example='node instance with the same '
-                                    'deployment ID',
-                            fields='|'.join(NODE_INSTANCES_SUMMARY_FIELDS)))
+                                   'details [manager only]')
 @cfy.argument('target_field',
-              type=cfy.SummaryArgs(NODE_INSTANCES_SUMMARY_FIELDS))
+              type=click.Choice(NODE_INSTANCES_SUMMARY_FIELDS))
 @cfy.argument('sub_field',
-              type=cfy.SummaryArgs(NODE_INSTANCES_SUMMARY_FIELDS),
+              type=click.Choice(NODE_INSTANCES_SUMMARY_FIELDS),
               default=None, required=False)
 @cfy.options.common_options
 @cfy.options.tenant_name(required=False, resource_name_for_help='summary')
@@ -163,6 +151,11 @@ def list(deployment_id,
 @cfy.pass_client()
 def summary(target_field, sub_field, logger, client, tenant_name,
             all_tenants):
+    """Retrieve summary of node instances, e.g. a count of each node instance
+    with the same deployment ID.
+
+    `TARGET_FIELD` is the field to summarise node instances on.
+    """
     utils.explicit_tenant_name_message(tenant_name, logger)
     logger.info(
         'Retrieving summary of node instances on field {field}'.format(
@@ -200,7 +193,6 @@ def summary(target_field, sub_field, logger, client, tenant_name,
                          resource_name_for_help='node-instance')
 @cfy.pass_logger
 @cfy.pass_client()
-@cfy.options.extended_view
 def update_runtime(node_instance_id, logger, client, tenant_name, properties):
     """Update the runtime properties of a specific node-instance
 
@@ -220,7 +212,6 @@ def update_runtime(node_instance_id, logger, client, tenant_name, properties):
                          resource_name_for_help='node-instance')
 @cfy.pass_logger
 @cfy.pass_client()
-@cfy.options.extended_view
 def delete_runtime(node_instance_id, logger, client, tenant_name, properties):
     """Delete specified runtime properties of a specific node-instance
 
@@ -272,11 +263,14 @@ def local(node_id, blueprint_id, logger):
 
 
 def _print_node_instance(node_instance):
+    # Decode in-place so that it's decoded for both branches
+    node_instance.runtime_properties.update(
+        utils.decode_dict(node_instance.runtime_properties))
+
     if get_global_json_output():
-        # For json output, make sure the properties are in the same object
+        # For json output, make sure runtime properties are in the same object
         # so that the output is a single decode-able object
-        columns = NODE_INSTANCE_COLUMNS + \
-            ['runtime_properties', 'system_properties']
+        columns = NODE_INSTANCE_COLUMNS + ['runtime_properties']
         print_single(columns, node_instance, 'Node-instance:', 50)
     else:
         print_single(NODE_INSTANCE_COLUMNS, node_instance,
@@ -284,7 +278,3 @@ def _print_node_instance(node_instance):
 
         print_details(node_instance.runtime_properties,
                       'Instance runtime properties:')
-
-        if hasattr(node_instance, 'system_properties'):
-            print_details(node_instance.system_properties,
-                          'Instance system properties:')
