@@ -17,19 +17,12 @@
 import click
 from cloudify_rest_client.exceptions import CloudifyClientError
 
-from cloudify_cli import utils
-from cloudify_cli.cli import cfy, helptexts
-from cloudify_cli.exceptions import CloudifyCliError
-from cloudify_cli.logger import (
-    get_events_logger,
-    get_global_json_output,
-    get_global_extended_view,
-)
-from cloudify_cli.table import print_data, print_single, print_details
-from cloudify_cli.commands.summary import (
-    BASE_SUMMARY_FIELDS,
-    structure_summary_results)
-from cloudify_cli.execution_events_fetcher import wait_for_execution
+from .. import utils
+from ..cli import cfy
+from ..table import print_data, print_single, print_details
+from ..exceptions import CloudifyCliError
+from ..logger import get_global_json_output
+from .summary import BASE_SUMMARY_FIELDS, structure_summary_results
 
 NODE_COLUMNS = ['id', 'deployment_id', 'blueprint_id', 'host_id', 'type',
                 'visibility', 'tenant_name', 'actual_number_of_instances',
@@ -60,18 +53,9 @@ def nodes():
 @cfy.options.deployment_id(required=True)
 @cfy.options.common_options
 @cfy.options.tenant_name(required=False, resource_name_for_help='node')
-@cfy.options.evaluate_functions
 @cfy.pass_logger
 @cfy.pass_client()
-@cfy.options.extended_view
-def get(
-    node_id,
-    deployment_id,
-    evaluate_functions,
-    client,
-    logger,
-    tenant_name,
-):
+def get(node_id, deployment_id, logger, client, tenant_name):
     """Retrieve information for a specific node of a specific deployment
 
     `NODE_ID` is the node id to get information on.
@@ -80,11 +64,7 @@ def get(
     logger.info('Retrieving node {0} for deployment {1}'.format(
         node_id, deployment_id))
     try:
-        node = client.nodes.get(
-            deployment_id,
-            node_id,
-            evaluate_functions=evaluate_functions,
-        )
+        node = client.nodes.get(deployment_id, node_id)
     except CloudifyClientError as e:
         if e.status_code != 404:
             raise
@@ -119,7 +99,7 @@ def get(
         print_details(node.properties, 'Node properties:')
 
         operations = []
-        for op_name, op in node.operations.items():
+        for op_name, op in utils.decode_dict(node.operations).iteritems():
             # operations is a tuple (operation_name, dict_of_attributes)
             # we want to add the name to the dict
             # and build a new array in order to print it in a table
@@ -136,21 +116,6 @@ def get(
             logger.info('\tNo node instances')
 
 
-def _run_node_checks(deployment_id, logger, client):
-    events_logger = get_events_logger()
-    for workflow_id in ('check_drift', 'check_status'):
-        execution = client.executions.start(
-            deployment_id=deployment_id,
-            workflow_id=workflow_id,
-        )
-        wait_for_execution(
-            client,
-            execution,
-            events_handler=events_logger,
-            logger=logger,
-        )
-
-
 @nodes.command(name='list',
                short_help='List nodes for a deployment '
                '[manager only]')
@@ -159,62 +124,42 @@ def _run_node_checks(deployment_id, logger, client):
 @cfy.options.descending
 @cfy.options.tenant_name_for_list(
     required=False, resource_name_for_help='node')
-@click.option(
-    '--run-checks',
-    help='Run the check_drift and check_status workflows before listing nodes',
-    is_flag=True,
-)
 @cfy.options.all_tenants
 @cfy.options.search
 @cfy.options.pagination_offset
 @cfy.options.pagination_size
 @cfy.options.common_options
-@cfy.options.evaluate_functions
 @cfy.pass_logger
 @cfy.pass_client()
-@cfy.options.extended_view
-def nodes_list(
-    deployment_id,
-    sort_by,
-    descending,
-    tenant_name,
-    all_tenants,
-    run_checks,
-    search,
-    pagination_offset,
-    pagination_size,
-    evaluate_functions,
-    logger,
-    client
-):
+def list(deployment_id,
+         sort_by,
+         descending,
+         tenant_name,
+         all_tenants,
+         search,
+         pagination_offset,
+         pagination_size,
+         logger, client):
     """List nodes
 
     If `DEPLOYMENT_ID` is provided, list nodes for that deployment.
     Otherwise, list nodes for all deployments.
     """
     utils.explicit_tenant_name_message(tenant_name, logger)
-    if run_checks:
-        if not deployment_id:
-            raise CloudifyCliError(
-                "deployment_id is required when --run-checks is set")
-        _run_node_checks(deployment_id, logger, client)
     try:
         if deployment_id:
-            logger.info('Listing nodes for deployment %s...', deployment_id)
-            instance_counts = True
+            logger.info('Listing nodes for deployment {0}...'.format(
+                deployment_id))
         else:
             logger.info('Listing all nodes...')
-            instance_counts = False
         nodes = client.nodes.list(
             deployment_id=deployment_id,
             sort=sort_by,
             is_descending=descending,
-            evaluate_functions=evaluate_functions,
             _all_tenants=all_tenants,
             _search=search,
             _offset=pagination_offset,
-            _size=pagination_size,
-            _instance_counts=instance_counts,
+            _size=pagination_size
         )
     except CloudifyClientError as e:
         if e.status_code != 404:
@@ -222,25 +167,15 @@ def nodes_list(
         raise CloudifyCliError('Deployment {0} does not exist'.format(
             deployment_id))
 
-    columns = list(NODE_COLUMNS)
-    if (
-        (get_global_json_output() or get_global_extended_view())
-        and deployment_id
-    ):
-        columns = columns + ['drifted_instances', 'unavailable_instances']
-    print_data(columns, nodes, 'Nodes:', labels=NODE_TABLE_LABELS)
+    print_data(NODE_COLUMNS, nodes, 'Nodes:', labels=NODE_TABLE_LABELS)
     total = nodes.metadata.pagination.total
-    logger.info('Showing %d of %d nodes', len(nodes), total)
+    logger.info('Showing {0} of {1} nodes'.format(len(nodes), total))
 
 
 @nodes.command(name='summary',
-               short_help='Retrieve summary of node details [manager only]',
-               help=helptexts.SUMMARY_HELP.format(
-                   type='nodes',
-                   example='node with the same deployment ID',
-                   fields='|'.join(NODES_SUMMARY_FIELDS)))
-@cfy.argument('target_field', type=cfy.SummaryArgs(NODES_SUMMARY_FIELDS))
-@cfy.argument('sub_field', type=cfy.SummaryArgs(NODES_SUMMARY_FIELDS),
+               short_help='Retrieve summary of node details [manager only]')
+@cfy.argument('target_field', type=click.Choice(NODES_SUMMARY_FIELDS))
+@cfy.argument('sub_field', type=click.Choice(NODES_SUMMARY_FIELDS),
               default=None, required=False)
 @cfy.options.common_options
 @cfy.options.tenant_name(required=False, resource_name_for_help='summary')
@@ -248,6 +183,11 @@ def nodes_list(
 @cfy.pass_logger
 @cfy.pass_client()
 def summary(target_field, sub_field, logger, client, tenant_name, all_tenants):
+    """Retrieve summary of nodes, e.g. a count of each node with the same
+    deployment ID.
+
+    `TARGET_FIELD` is the field to summarise nodes on.
+    """
     utils.explicit_tenant_name_message(tenant_name, logger)
     logger.info('Retrieving summary of nodes on field {field}'.format(
         field=target_field))
