@@ -1,3 +1,19 @@
+########
+# Copyright (c) 2014 GigaSpaces Technologies Ltd. All rights reserved
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+############
+
 import os
 import shutil
 import tarfile
@@ -9,33 +25,33 @@ from cloudify.cluster_status import CloudifyNodeType
 from cloudify_rest_client.exceptions import (CloudifyClientError,
                                              UserUnauthorizedError)
 
-from cloudify_cli import constants, env, utils
-from cloudify_cli.cli import cfy, helptexts
-from cloudify_cli.env import get_rest_client
-from cloudify_cli.exceptions import CloudifyCliError
-from cloudify_cli.table import print_data, print_single
-from cloudify_cli.commands import init
-from cloudify_cli.commands.cluster import (
-    _all_in_one_manager,
-    update_profile_logic as update_cluster_profile)
+from . import init
+from .. import env
+from .. import utils
+from ..cli import cfy
+from .. import constants
+from ..cli import helptexts
+from ..env import get_rest_client
+from ..exceptions import CloudifyCliError
+from ..table import print_data, print_single
+from ..commands.cluster import _all_in_one_manager
+from ..commands.cluster import update_profile_logic as update_cluster_profile
 
 
 EXPORTED_KEYS_DIRNAME = '.exported-ssh-keys'
+EXPORTED_SSH_KEYS_DIR = os.path.join(env.PROFILES_DIR, EXPORTED_KEYS_DIRNAME)
 PROFILE_COLUMNS = ['name', 'manager_ip', 'manager_username', 'manager_tenant',
-                   'ssh_user', 'ssh_key', 'ssh_port', 'kerberos_env',
+                   'ssh_user', 'ssh_key_path', 'ssh_port', 'kerberos_env',
                    'rest_port', 'rest_protocol', 'rest_certificate']
 CLUSTER_PROFILE_COLUMNS = PROFILE_COLUMNS[:1] + ['hostname', 'host_ip'] \
     + PROFILE_COLUMNS[2:]
 
 
-def _exported_ssh_keys_dir():
-    return os.path.join(env.PROFILES_DIR, EXPORTED_KEYS_DIRNAME)
-
-
 @cfy.group(name='profiles')
 @cfy.options.common_options
 def profiles():
-    """Handle Cloudify CLI profiles
+    """
+    Handle Cloudify CLI profiles
 
     Each profile can manage a single Cloudify manager.
 
@@ -69,7 +85,6 @@ def _format_cluster_profile(profile):
                   short_help='Retrieve current profile information')
 @cfy.options.common_options
 @cfy.pass_logger
-@cfy.options.extended_view
 def show(logger):
     """
     Shows your current active profile and it's properties
@@ -99,8 +114,7 @@ def show(logger):
                   short_help='List profiles')
 @cfy.options.common_options
 @cfy.pass_logger
-@cfy.options.extended_view
-def profiles_list(logger):
+def list(logger):
     """
     List all profiles
     """
@@ -132,10 +146,9 @@ def profiles_list(logger):
 @cfy.options.ssh_user
 @cfy.options.ssh_key
 @cfy.options.ssh_port
-@cfy.options.manager_token
 @cfy.options.manager_username
 @cfy.options.manager_password
-@cfy.options.manager_tenant()
+@cfy.options.manager_tenant
 @cfy.options.rest_port
 @cfy.options.ssl_rest
 @cfy.options.rest_certificate
@@ -155,13 +168,6 @@ def use(manager_ip,
     Additional CLI commands will be added after a manager is used.
     To stop using a manager, you can run `cfy init -r`.
     """
-    def _auth_args_present(kwargs):
-        return any(
-            kwargs.get(kwarg)
-            for kwarg in ['manager_username', 'manager_password',
-                          'manager_token', 'kerberos_env']
-        )
-
     if not profile_name:
         profile_name = manager_ip
     if profile_name == 'local':
@@ -178,30 +184,19 @@ def use(manager_ip,
             logger=logger,
             **kwargs)
     else:
-        kwargs.setdefault('manager_tenant', 'default_tenant')
-        if _auth_args_present(kwargs):
-            env.check_configured_auth(
-                credentials=(kwargs['manager_username'],
-                             kwargs['manager_password']),
-                token=kwargs['manager_token'],
-                kerberos_env=kwargs['kerberos_env'],
-            )
         _create_profile(
             manager_ip=manager_ip,
             profile_name=profile_name,
             skip_credentials_validation=skip_credentials_validation,
             logger=logger,
             **kwargs)
-    if (
-        # We test with the env here in case of switching to an existing one
-        _auth_args_present(env.profile.to_dict())
-        and not skip_credentials_validation
-    ):
-        _update_cluster_profile_to_dict(logger)
+    if not env.profile.manager_username:
+        return
+    _update_cluster_profile_to_dict(logger)
 
 
 def _update_cluster_profile_to_dict(logger):
-    if isinstance(env.profile.cluster, list):   # noqa
+    if type(env.profile.cluster) == type([]):   # noqa
         env.profile.cluster = dict()
         env.profile.save()
     client = get_rest_client()
@@ -212,18 +207,15 @@ def _update_cluster_profile_to_dict(logger):
 def _switch_profile(manager_ip, profile_name, logger, **kwargs):
     # if using an existing profile, it is an error to provide any --option,
     # because the way to update an existing profile is `cfy profiles set`
-    if kwargs.get('manager_tenant') == env.profile.manager_tenant:
-        del kwargs['manager_tenant']
     provided_options = [key for key, value in kwargs.items() if value]
-
     if any(provided_options):
-        logger.warning('Profile %s already exists. '
-                       'The passed in options are ignored: %s. '
-                       'To update the profile, use `cfy profiles set`',
-                       profile_name, ', '.join(provided_options))
+        logger.warning('Profile {0} already exists. '
+                       'The passed in options are ignored: {1}. '
+                       'To update the profile, use `cfy profiles set`'
+                       .format(profile_name, ', '.join(provided_options)))
 
     env.set_active_profile(profile_name)
-    logger.info('Using manager %s', profile_name)
+    logger.info('Using manager {0}'.format(profile_name))
 
 
 def _create_profile(
@@ -235,7 +227,6 @@ def _create_profile(
         manager_username,
         manager_password,
         manager_tenant,
-        manager_token,
         rest_port,
         ssl,
         rest_certificate,
@@ -253,25 +244,41 @@ def _create_profile(
     # kerberos_env default is `False` and not `None`
     kerberos_env = get_kerberos_indication(kerberos_env) or False
 
-    logger.info('Attempting to connect to %s through port %s, using %s '
-                '(SSL mode: %s)...', manager_ip, rest_port, rest_protocol, ssl)
+    logger.info('Attempting to connect to {0} through port {1}, using {2} '
+                '(SSL mode: {3})...'.format(manager_ip, rest_port,
+                                            rest_protocol, ssl))
 
-    logger.info('Using manager %s with port %s', manager_ip, rest_port)
+    # First, attempt to get the provider from the manager - should it fail,
+    # the manager's profile directory won't be created
+    provider_context = _get_provider_context(
+        profile_name,
+        manager_ip,
+        rest_port,
+        rest_protocol,
+        rest_certificate,
+        manager_username,
+        manager_password,
+        manager_tenant,
+        kerberos_env,
+        skip_credentials_validation
+    )
+    init.init_manager_profile(profile_name=profile_name)
+    logger.info('Using manager {0} with port {1}'.format(
+        manager_ip, rest_port))
     _set_profile_context(
         profile_name,
+        provider_context,
         manager_ip,
         ssh_key,
         ssh_user,
         ssh_port,
         manager_username,
         manager_password,
-        manager_token,
         manager_tenant,
         rest_port,
         rest_protocol,
         rest_certificate,
-        kerberos_env,
-        skip_credentials_validation,
+        kerberos_env
     )
     env.set_active_profile(profile_name)
 
@@ -286,12 +293,97 @@ def delete(profile_name, logger):
 
     `PROFILE_NAME` is the IP of the manager the profile manages.
     """
-    logger.info('Deleting profile %s...', profile_name)
-    if not env.is_profile_exists(profile_name):
-        raise CloudifyCliError('Profile {0} does not exist'
-                               .format(profile_name))
-    env.delete_profile(profile_name)
-    logger.info('Profile deleted')
+    logger.info('Deleting profile {0}...'.format(profile_name))
+    try:
+        env.delete_profile(profile_name)
+        logger.info('Profile deleted')
+    except CloudifyCliError as ex:
+        logger.info(str(ex))
+
+
+def set_profile(profile_name,
+                manager_username,
+                manager_password,
+                manager_tenant,
+                ssh_user,
+                ssh_key,
+                ssh_port,
+                ssl,
+                rest_certificate,
+                rest_port,
+                kerberos_env,
+                skip_credentials_validation,
+                logger):
+    """Set the profile name, manager username and/or password and/or tenant
+    and/or ssl state (on/off) in the *current* profile
+    """
+    if not any([profile_name, ssh_user, ssh_key, ssh_port, manager_username,
+                manager_password, manager_tenant, ssl is not None,
+                rest_certificate, kerberos_env is not None]):
+        raise CloudifyCliError(
+            "You must supply at least one of the following:  "
+            "profile name, username, password, tenant, "
+            "ssl, rest certificate, ssh user, ssh key, ssh port, kerberos env")
+    username = manager_username or env.get_username()
+    password = manager_password or env.get_password()
+    tenant = manager_tenant or env.get_tenant_name()
+    protocol, port = _get_ssl_protocol_and_port(ssl)
+    if rest_port is not None:
+        port = rest_port
+
+    if not skip_credentials_validation:
+        _validate_credentials(username,
+                              password,
+                              tenant,
+                              rest_certificate,
+                              protocol,
+                              port,
+                              kerberos_env)
+    old_name = None
+    if profile_name:
+        if profile_name == 'local':
+            raise CloudifyCliError('Cannot use the reserved name "local"')
+        if env.is_profile_exists(profile_name):
+            raise CloudifyCliError('Profile {0} already exists'
+                                   .format(profile_name))
+        old_name = env.profile.profile_name
+        env.profile.profile_name = profile_name
+    if manager_username:
+        logger.info('Setting username to `{0}`'.format(manager_username))
+        env.profile.manager_username = manager_username
+    if manager_password:
+        logger.info('Setting password to `{0}`'.format(manager_password))
+        env.profile.manager_password = manager_password
+    if manager_tenant:
+        logger.info('Setting tenant to `{0}`'.format(manager_tenant))
+        env.profile.manager_tenant = manager_tenant
+    if rest_certificate:
+        logger.info(
+            'Setting rest certificate to `{0}`'.format(rest_certificate))
+        env.profile.rest_certificate = rest_certificate
+    if rest_port:
+        logger.info('Setting rest port to `{0}'.format(rest_port))
+        env.profile.rest_port = rest_port
+    if ssh_user:
+        logger.info('Setting ssh user to `{0}`'.format(ssh_user))
+        env.profile.ssh_user = ssh_user
+    if ssh_key:
+        logger.info('Setting ssh key to `{0}`'.format(ssh_key))
+        env.profile.ssh_key = ssh_key
+    if ssh_port:
+        logger.info('Setting ssh port to `{0}`'.format(ssh_port))
+        env.profile.ssh_port = ssh_port
+    if kerberos_env is not None:
+        logger.info('Setting kerberos_env to `{0}`'.format(kerberos_env))
+        env.profile.kerberos_env = kerberos_env
+    if ssl is not None:
+        _set_profile_ssl(ssl, rest_port, logger)
+
+    env.profile.save()
+    if old_name is not None:
+        env.set_active_profile(profile_name)
+        env.delete_profile(old_name)
+    logger.info('Settings saved successfully')
 
 
 def _set_profile_ssl(ssl, rest_port, logger):
@@ -316,12 +408,12 @@ def _set_profile_ssl(ssl, rest_port, logger):
         for node in manager_cluster:
             node['rest_port'] = port
             node['rest_protocol'] = protocol
-            logger.info('Enabling SSL for %(host_ip)s', node)
+            logger.info('Enabling SSL for {0}'.format(node['host_ip']))
             if not node.get('cert'):
                 missing_certs.append(node['hostname'])
         if missing_certs:
             logger.warning('The following cluster nodes have no certificate '
-                           'set: %s', ', '.join(missing_certs))
+                           'set: {0}'.format(', '.join(missing_certs)))
             logger.warning('If required, set the certificates for those '
                            'nodes using `cfy profiles set-cluster`')
 
@@ -330,11 +422,9 @@ def _set_profile_ssl(ssl, rest_port, logger):
     name='set',
     short_help='Set name/manager username/password/tenant in current profile')
 @cfy.options.profile_name
-@cfy.options.profile_manager_ip
-@cfy.options.manager_token
 @cfy.options.manager_username
 @cfy.options.manager_password
-@cfy.options.manager_tenant()
+@cfy.options.manager_tenant
 @cfy.options.ssh_user
 @cfy.options.ssh_key
 @cfy.options.ssh_port
@@ -346,8 +436,6 @@ def _set_profile_ssl(ssl, rest_port, logger):
 @cfy.options.common_options
 @cfy.pass_logger
 def set_cmd(profile_name,
-            manager_ip,
-            manager_token,
             manager_username,
             manager_password,
             manager_tenant,
@@ -360,83 +448,19 @@ def set_cmd(profile_name,
             kerberos_env,
             skip_credentials_validation,
             logger):
-    """Set the profile name, manager username and/or password and/or tenant
-    and/or ssl state (on/off) in the *current* profile
-    """
-    if not any([profile_name, manager_ip, ssh_user, ssh_key, ssh_port,
-                manager_token, manager_username, manager_password,
-                manager_tenant, ssl is not None, rest_certificate,
-                kerberos_env is not None]):
-        raise CloudifyCliError(
-            "You must supply at least one of the following:  "
-            "profile name, username, password, token, tenant, "
-            "ssl, rest certificate, ssh user, ssh key, ssh port, kerberos env")
-    old_name = None
-    if profile_name:
-        if profile_name == 'local':
-            raise CloudifyCliError('Cannot use the reserved name "local"')
-        if env.is_profile_exists(profile_name):
-            raise CloudifyCliError('Profile {0} already exists'
-                                   .format(profile_name))
-        old_name = env.profile.profile_name
-        env.profile.profile_name = profile_name
-    if manager_ip:
-        env.profile.manager_ip = manager_ip
-        logger.info('Setting the manager address to `%s`', manager_ip)
-    if manager_username:
-        logger.info('Setting username to `%s`', manager_username)
-        env.profile.manager_username = manager_username
-        logger.info('Clearing non-credentials auth')
-        env.profile.manager_token = None
-        env.profile.kerberos_env = None
-    if manager_password:
-        logger.info('Setting password')
-        env.profile.manager_password = manager_password
-    if manager_token:
-        logger.info('Setting token')
-        logger.info('Clearing non-token auth')
-        env.profile.manager_token = manager_token
-        env.profile.manager_username = None
-        env.profile.manager_password = None
-        env.profile.kerberos_env = None
-    if manager_tenant:
-        logger.info('Setting tenant to `%s`', manager_tenant)
-        env.profile.manager_tenant = manager_tenant
-    if rest_certificate:
-        logger.info('Setting rest certificate to `%s`', rest_certificate)
-        env.profile.rest_certificate = rest_certificate
-    if rest_port:
-        logger.info('Setting rest port to `%s', rest_port)
-        env.profile.rest_port = rest_port
-    if ssh_user:
-        logger.info('Setting ssh user to `%s`', ssh_user)
-        env.profile.ssh_user = ssh_user
-    if ssh_key:
-        logger.info('Setting ssh key to `%s`', ssh_key)
-        env.profile.ssh_key = ssh_key
-    if ssh_port:
-        logger.info('Setting ssh port to `%s`', ssh_port)
-        env.profile.ssh_port = ssh_port
-    if kerberos_env is not None:
-        logger.info('Setting kerberos_env to `%s`', kerberos_env)
-        logger.info('Clearing non-kerberos auth')
-        env.profile.kerberos_env = kerberos_env
-        env.profile.manager_username = None
-        env.profile.manager_password = None
-        env.profile.manager_token = None
-    if ssl is not None:
-        _set_profile_ssl(ssl, rest_port, logger)
-
-    if not skip_credentials_validation:
-        _validate_credentials(env.profile)
-
-    env.profile.save()
-    if old_name is not None:
-        env.set_active_profile(profile_name)
-        env.delete_profile(old_name)
-    logger.info('Settings saved successfully')
-    if not skip_credentials_validation:
-        _update_cluster_profile_to_dict(logger)
+    return set_profile(profile_name,
+                       manager_username,
+                       manager_password,
+                       manager_tenant,
+                       ssh_user,
+                       ssh_key,
+                       ssh_port,
+                       _get_ssl_indication(ssl),
+                       rest_certificate,
+                       rest_port,
+                       get_kerberos_indication(kerberos_env),
+                       skip_credentials_validation,
+                       logger)
 
 
 @profiles.command(
@@ -483,14 +507,15 @@ def set_cluster(cluster_node_name,
     ]:
         if source:
             changed_node[target] = source
-            logger.info('Node %s: setting %s to `%s`',
-                        cluster_node_name, label, source)
+            logger.info('Node {0}: setting {1} to `{2}`'
+                        .format(cluster_node_name, label, source))
     if rest_certificate:
         changed_node['cert'] = rest_certificate
         changed_node['trust_all'] = False
         changed_node['rest_protocol'] = 'https'
-        logger.info('Node %s: setting rest-certificate to `%s` and enabling '
-                    'certificate verification', cluster_node_name, source)
+        logger.info('Node {0}: setting rest-certificate to `{1}` and enabling '
+                    'certificate verification'
+                    .format(cluster_node_name, source))
     env.profile.save()
     logger.info('Settings saved successfully')
 
@@ -525,6 +550,33 @@ def unset(manager_username,
         raise CloudifyCliError("You must choose at least one of the following:"
                                " username, password, tenant, kerberos_env, "
                                "rest certificate, ssh user, ssh key")
+    if manager_username:
+        username = os.environ.get(constants.CLOUDIFY_USERNAME_ENV)
+    else:
+        username = env.profile.manager_username
+    if manager_password:
+        password = os.environ.get(constants.CLOUDIFY_PASSWORD_ENV)
+    else:
+        password = env.profile.manager_password
+    if manager_tenant:
+        tenant = os.environ.get(constants.CLOUDIFY_TENANT_ENV)
+    else:
+        tenant = env.profile.manager_tenant
+    if rest_certificate:
+        cert = os.environ.get(constants.LOCAL_REST_CERT_FILE) \
+            or env.get_default_rest_cert_local_path()
+    else:
+        cert = None
+
+    if not skip_credentials_validation:
+        _validate_credentials(username,
+                              password,
+                              tenant,
+                              cert,
+                              env.profile.rest_protocol,
+                              env.profile.rest_port,
+                              None)
+
     if manager_username:
         logger.info('Clearing manager username')
         env.profile.manager_username = None
@@ -546,10 +598,6 @@ def unset(manager_username,
     if kerberos_env:
         logger.info('Clearing kerberos_env')
         env.profile.kerberos_env = None
-
-    if not skip_credentials_validation:
-        _validate_credentials(env.profile)
-
     env.profile.save()
     logger.info('Settings saved successfully')
 
@@ -576,13 +624,13 @@ def export_profiles(include_keys, output_path, logger):
         os.path.join(os.getcwd(), 'cfy-profiles.tar.gz')
 
     # TODO: Copy exported ssh keys to each profile's directory
-    logger.info('Exporting profiles to %s...', destination)
+    logger.info('Exporting profiles to {0}...'.format(destination))
     if include_keys:
         for profile in env.get_profile_names():
             _backup_ssh_key(profile)
     utils.tar(env.PROFILES_DIR, destination)
     if include_keys:
-        shutil.rmtree(_exported_ssh_keys_dir())
+        shutil.rmtree(EXPORTED_SSH_KEYS_DIR)
     logger.info('Export complete!')
     logger.info(
         'You can import the profiles by running '
@@ -606,7 +654,7 @@ def import_profiles(archive_path, include_keys, logger):
     _assert_is_tarfile(archive_path)
     _assert_profiles_archive(archive_path)
 
-    logger.info('Importing profiles from %s...', archive_path)
+    logger.info('Importing profiles from {0}...'.format(archive_path))
     utils.untar(archive_path, os.path.dirname(env.PROFILES_DIR))
 
     if include_keys:
@@ -617,8 +665,8 @@ def import_profiles(archive_path, include_keys, logger):
             logger.info("The profiles archive you provided contains ssh keys "
                         "for one or more profiles. To restore those keys to "
                         "their original locations, you can use the "
-                        "`--include-keys flag or copy them manually from %s ",
-                        _exported_ssh_keys_dir())
+                        "`--include-keys flag or copy them manually from {0} "
+                        .format(EXPORTED_SSH_KEYS_DIR))
     logger.info('Import complete!')
     logger.info('You can list profiles using `cfy profiles list`')
 
@@ -659,18 +707,19 @@ def _move_ssh_key(profile, logger, is_backup):
     key_filepath = context.ssh_key
     if key_filepath:
         backup_path = os.path.join(
-            _exported_ssh_keys_dir(), os.path.basename(key_filepath)) + \
+            EXPORTED_SSH_KEYS_DIR, os.path.basename(key_filepath)) + \
             '.{0}.profile'.format(profile)
         if is_backup:
-            if not os.path.isdir(_exported_ssh_keys_dir()):
-                os.makedirs(_exported_ssh_keys_dir(), mode=0o700)
-            logger.info('Copying ssh key %s to %s...',
-                        key_filepath, backup_path)
+            if not os.path.isdir(EXPORTED_SSH_KEYS_DIR):
+                os.makedirs(EXPORTED_SSH_KEYS_DIR)
+            logger.info('Copying ssh key {0} to {1}...'.format(
+                key_filepath, backup_path))
             shutil.copy2(key_filepath, backup_path)
         else:
             if os.path.isfile(backup_path):
-                logger.info('Restoring ssh key for profile %s to %s...',
-                            profile, key_filepath)
+                logger.info(
+                    'Restoring ssh key for profile {0} to {1}...'.format(
+                        profile, key_filepath))
                 shutil.move(backup_path, key_filepath)
 
 
@@ -686,7 +735,7 @@ def _get_profile(profile_name):
 def _assert_manager_available(client, profile_name):
     try:
         return client.manager.get_status()
-    except UserUnauthorizedError as e:
+    except UserUnauthorizedError, e:
         raise CloudifyCliError(
             "Can't use manager {0}\n{1}.".format(
                 profile_name,
@@ -699,12 +748,31 @@ def _assert_manager_available(client, profile_name):
     # with situations like No route to host and the likes.
     except Exception as ex:
         raise CloudifyCliError(
-            "Can't use manager {0}. {1}".format(profile_name, ex))
+            "Can't use manager {0}. {1}".format(profile_name, str(ex.message)))
 
 
-def _get_provider_context(profile, skip_credentials_validation):
+def _get_provider_context(profile_name,
+                          manager_ip,
+                          rest_port,
+                          rest_protocol,
+                          rest_certificate,
+                          manager_username,
+                          manager_password,
+                          manager_tenant,
+                          kerberos_env,
+                          skip_credentials_validation):
     try:
-        client = _get_client_and_assert_manager(profile=profile)
+        client = _get_client_and_assert_manager(
+            profile_name,
+            manager_ip,
+            rest_port,
+            rest_protocol,
+            rest_certificate,
+            manager_username,
+            manager_password,
+            manager_tenant,
+            kerberos_env
+        )
     except CloudifyCliError:
         if skip_credentials_validation:
             return None
@@ -717,31 +785,51 @@ def _get_provider_context(profile, skip_credentials_validation):
         return None
 
 
-def _get_client_and_assert_manager(profile):
-    if not profile.manager_ip:
-        raise CloudifyCliError('No manager IP defined for Cloudify CLI '
-                               'usage.\nPlease define a profile using '
-                               '`cfy profiles use`')
-    client = env.get_rest_client(client_profile=profile)
-    _assert_manager_available(client, profile.name)
+def _get_client_and_assert_manager(profile_name,
+                                   manager_ip=None,
+                                   rest_port=None,
+                                   rest_protocol=None,
+                                   rest_certificate=None,
+                                   manager_username=None,
+                                   manager_password=None,
+                                   manager_tenant=None,
+                                   kerberos_env=None):
+    # Attempt to update the profile with an existing profile context, if one
+    # is available. This is relevant in case the user didn't pass a username
+    # or a password, and was expecting them to be taken from the old profile
+    env.profile = env.get_profile_context(profile_name, suppress_error=True)
+
+    client = env.get_rest_client(
+        rest_host=manager_ip,
+        rest_port=rest_port,
+        rest_protocol=rest_protocol,
+        rest_cert=rest_certificate,
+        username=manager_username,
+        password=manager_password,
+        tenant_name=manager_tenant,
+        kerberos_env=kerberos_env
+    )
+
+    _assert_manager_available(client, profile_name)
     return client
 
 
 def _set_profile_context(profile_name,
+                         provider_context,
                          manager_ip,
                          ssh_key,
                          ssh_user,
                          ssh_port,
                          manager_username,
                          manager_password,
-                         manager_token,
                          manager_tenant,
                          rest_port,
                          rest_protocol,
                          rest_certificate,
-                         kerberos_env,
-                         skip_credentials_validation):
-    profile = env.ProfileContext()
+                         kerberos_env):
+
+    profile = env.get_profile_context(profile_name)
+    profile.provider_context = provider_context
     if profile_name:
         profile.profile_name = profile_name
     if manager_ip:
@@ -756,8 +844,6 @@ def _set_profile_context(profile_name,
         profile.manager_username = manager_username
     if manager_password:
         profile.manager_password = manager_password
-    if manager_token:
-        profile.manager_token = manager_token
     if manager_tenant:
         profile.manager_tenant = manager_tenant
     profile.ssh_port = ssh_port or constants.REMOTE_EXECUTION_PORT
@@ -765,11 +851,7 @@ def _set_profile_context(profile_name,
     profile.rest_certificate = rest_certificate
     profile.kerberos_env = kerberos_env
 
-    profile.provider_context = _get_provider_context(
-        profile, skip_credentials_validation)
-    # We initialise the profile here so that the profile dir doesn't get
-    # created if we unexpectedly fail to get the provider context
-    init.init_manager_profile(profile_name=profile_name, profile=profile)
+    profile.save()
 
 
 def _is_manager_secured(response_history):
@@ -786,6 +868,12 @@ def _is_manager_secured(response_history):
     return False
 
 
+def _get_ssl_indication(ssl):
+    if ssl is None:
+        return None
+    return str(ssl).lower() == 'on'
+
+
 def _get_ssl_protocol_and_port(ssl):
     if ssl is not None:
         protocol, port = (constants.SECURED_REST_PROTOCOL,
@@ -797,7 +885,17 @@ def _get_ssl_protocol_and_port(ssl):
 
 
 @cfy.pass_logger
-def _validate_credentials(profile, logger):
+def _validate_credentials(username, password, tenant, certificate, protocol,
+                          rest_port, kerberos_env, logger):
     logger.info('Validating credentials...')
-    _get_client_and_assert_manager(profile=profile)
+    _get_client_and_assert_manager(
+        profile_name=env.profile.profile_name,
+        manager_username=username,
+        manager_password=password,
+        manager_tenant=tenant,
+        rest_certificate=certificate,
+        rest_protocol=protocol,
+        rest_port=rest_port,
+        kerberos_env=kerberos_env
+    )
     logger.info('Credentials validated')
