@@ -15,10 +15,11 @@
 ############
 
 
-from cloudify_cli import env, utils
-from cloudify_cli.cli import cfy
-from cloudify_cli.table import print_data, print_single
-from cloudify_cli.utils import handle_client_error, validate_visibility
+from .. import env
+from .. import utils
+from ..cli import cfy
+from ..table import print_data, print_single
+from ..utils import handle_client_error, validate_visibility
 
 SITES_COLUMNS = ['name', 'location', 'visibility', 'tenant_name',
                  'created_at', 'created_by']
@@ -61,7 +62,6 @@ def create(name, location, visibility, tenant_name, client, logger):
 @cfy.assert_manager_active()
 @cfy.pass_client(use_tenant_in_header=True)
 @cfy.pass_logger
-@cfy.options.extended_view
 def get(name, tenant_name, client, logger):
     """Get details for a single site
 
@@ -112,7 +112,6 @@ def update(name, location, visibility, new_name, tenant_name, client, logger):
 @cfy.assert_manager_active()
 @cfy.pass_client()
 @cfy.pass_logger
-@cfy.options.extended_view
 def list(sort_by,
          descending,
          tenant_name,
