@@ -14,10 +14,10 @@
 # limitations under the License.
 ############
 
-from cloudify_cli import env
-from cloudify_cli.cli import cfy
-from cloudify_cli.table import print_data, print_single
-from cloudify_cli.utils import handle_client_error
+from .. import env
+from ..cli import cfy
+from ..table import print_data, print_single
+from ..utils import handle_client_error
 
 USER_COLUMNS = ['username', 'groups', 'role', 'group_system_roles', 'active',
                 'last_login_at', 'is_locked']
@@ -76,7 +76,6 @@ def users():
 @cfy.assert_manager_active()
 @cfy.pass_client()
 @cfy.pass_logger
-@cfy.options.extended_view
 def list(sort_by,
          descending,
          get_data,
@@ -110,7 +109,7 @@ def list(sort_by,
 
 
 @users.command(name='create', short_help='Create a user [manager only]')
-@cfy.argument('username')
+@cfy.argument('username', callback=cfy.validate_name)
 @cfy.options.common_options
 @cfy.options.security_role
 @cfy.options.password
@@ -132,25 +131,19 @@ def create(username,
     `USERNAME` is the username of the user
     """
     client.users.create(username, password, security_role)
-    logger.info('User `%s` created with `%s` security role',
-                username, security_role)
+    logger.info('User `{0}` created with `{1}` security role'.format(
+        username, security_role))
 
     if tenant_name and user_tenant_role:
         client.tenants.add_user(username, tenant_name, user_tenant_role)
         logger.info(
-            'User `%s` added successfully to tenant `%s` with `%s` role',
-            username, tenant_name, user_tenant_role)
-    elif user_tenant_role and not tenant_name:
-        logger.warning('User tenant role `%s` specified, but no tenant '
-                       'provided, ignoring the tenant role', user_tenant_role)
-    elif tenant_name and not user_tenant_role:
-        logger.warning('Tenant `%s` specified, but no user tenant role '
-                       'provided, ignoring the tenant', tenant_name)
+            'User `{0}` added successfully to tenant `{1}` with `{2}` role'
+            .format(username, tenant_name, user_tenant_role))
 
 
 @users.command(name='set-password',
                short_help='Set a new password for a user [manager only]')
-@cfy.argument('username')
+@cfy.argument('username', callback=cfy.validate_name)
 @cfy.options.password
 @cfy.options.common_options
 @cfy.assert_manager_active()
@@ -168,7 +161,7 @@ def set_password(username, password, logger, client):
 
 @users.command(name='set-role',
                short_help='Set a new role for a user [manager only]')
-@cfy.argument('username')
+@cfy.argument('username', callback=cfy.validate_name)
 @cfy.options.security_role
 @cfy.options.common_options
 @cfy.assert_manager_active()
@@ -187,13 +180,12 @@ def set_role(username, security_role, logger, client):
 @users.command(name='get',
                short_help='Get details for a single user [manager only]')
 @cfy.argument(
-    'username', default=env.get_username())
+    'username', callback=cfy.validate_name, default=env.get_username())
 @cfy.options.common_options
 @cfy.options.get_data
 @cfy.assert_manager_active()
 @cfy.pass_client()
 @cfy.pass_logger
-@cfy.options.extended_view
 def get(username, get_data, logger, client):
     """Get details for a single user
 
@@ -219,7 +211,7 @@ def get(username, get_data, logger, client):
 
 @users.command(name='delete',
                short_help='Delete a user [manager only]')
-@cfy.argument('username')
+@cfy.argument('username', callback=cfy.validate_name)
 @cfy.options.common_options
 @cfy.assert_manager_active()
 @cfy.pass_client()
