@@ -1,12 +1,29 @@
+########
+# Copyright (c) 2014 GigaSpaces Technologies Ltd. All rights reserved
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+############
+
+
 import os
 import json
-import errno
 import types
 import shutil
+import pkgutil
 import getpass
 import tempfile
 import itertools
-from base64 import b64encode
+from base64 import urlsafe_b64encode
 
 import yaml
 import requests
@@ -14,12 +31,11 @@ import requests
 from cloudify_rest_client import CloudifyClient
 from cloudify_rest_client.client import HTTPClient
 from cloudify.cluster_status import CloudifyNodeType
+from cloudify_rest_client.utils import is_kerberos_env
 from cloudify_rest_client.exceptions import CloudifyClientError
-from cloudify.utils import ipv6_url_compat
-
-from cloudify_cli import constants
-from cloudify_cli.exceptions import CloudifyCliError
 
+from . import constants
+from .exceptions import CloudifyCliError
 
 _ENV_NAME = 'manager'
 DEFAULT_LOG_FILE = os.path.expanduser(
@@ -37,16 +53,17 @@ CLUSTER_RETRY_INTERVAL = 5
 def delete_profile(profile_name):
     if is_profile_exists(profile_name):
         profile_dir = get_profile_dir(profile_name)
-        if profile_dir:
-            shutil.rmtree(profile_dir)
+        shutil.rmtree(profile_dir)
+    else:
+        raise CloudifyCliError(
+            'Profile {0} does not exist'.format(profile_name))
 
 
 def is_profile_exists(profile_name):
-    base_dir = get_profile_dir(profile_name)
-    if not base_dir:
+    try:
+        return os.path.isfile(get_context_path(profile_name))
+    except CloudifyCliError:
         return False
-    return (os.path.isfile(os.path.join(base_dir, 'context.json')) or
-            os.path.isfile(os.path.join(base_dir, 'context')))
 
 
 def assert_profile_exists(profile_name):
@@ -64,12 +81,11 @@ def set_active_profile(profile_name):
 
 
 def get_active_profile():
-    try:
+    if os.path.isfile(ACTIVE_PROFILE):
         with open(ACTIVE_PROFILE) as active_profile:
             return active_profile.read().strip()
-    except IOError as e:
-        if e.errno != errno.ENOENT:
-            raise
+    else:
+        # We return None explicitly as no profile is active.
         return None
 
 
@@ -83,6 +99,8 @@ def get_target_manager():
 
 
 def get_profile_names():
+    # TODO: This is too.. ambiguous. We should change it so there are
+    # no exclusions.
     excluded = ['local']
     profile_names = [item for item in os.listdir(PROFILES_DIR)
                      if item not in excluded and not item.startswith('.')]
@@ -105,65 +123,25 @@ def assert_local_active():
             'You can run `cfy profiles use local` to stop using a manager.')
 
 
-def check_configured_auth(credentials, token, kerberos_env, extra_help=''):
-    if all(item is None for item in credentials):
-        credentials = None
-
-    if credentials and None in credentials:
-        raise CloudifyCliError(
-            'If Manager Username is set, Manager Password must also be set.'
-        )
-
-    configured_auth = []
-    if kerberos_env:
-        configured_auth.append('kerberos')
-    if token:
-        configured_auth.append('token')
-    if credentials:
-        configured_auth.append('username+password')
-
-    if not configured_auth:
-        raise CloudifyCliError(
-            'At least one auth method must be set.\n' + extra_help
-        )
-    if len(configured_auth) > 1:
-        raise CloudifyCliError(
-            'Only one auth method may be set at a time.\n'
-            'You may need to unset env vars for auth such as '
-            'CLOUDIFY_USERNAME, CLOUDIFY_PASSWORD, CLOUDIFY_TOKEN, '
-            'or CLOUDIFY_KERBEROS_ENV\n'
-            '{extra_help}'.format(extra_help=extra_help)
-        )
-
-
-def assert_credentials_set(client_profile=None):
-    if client_profile is None:
-        client_profile = profile
+def assert_credentials_set():
     error_msg = 'Manager {0} must be set in order to use a manager.\n' \
                 'You can set it in the profile by running ' \
                 '`cfy profiles set {1}`, or you can set the `CLOUDIFY_{2}` ' \
                 'environment variable.'
-    if not get_tenant_name(client_profile):
+    if not get_kerberos_env():
+        if not get_username():
+            raise CloudifyCliError(
+                error_msg.format('Username', '--manager-username', 'USERNAME')
+            )
+        if not get_password():
+            raise CloudifyCliError(
+                error_msg.format('Password', '--manager-password', 'PASSWORD')
+            )
+    if not get_tenant_name():
         raise CloudifyCliError(
             error_msg.format('Tenant', '--manager-tenant', 'TENANT')
         )
 
-    kerberos_env = get_kerberos_env(client_profile)
-    token = get_token(client_profile)
-    credentials = (get_username(client_profile), get_password(client_profile))
-
-    configure_help = (
-        'Please configure either a token, kerberos, or a username+password. '
-        'Authentication may be configured with one of the following lines:\n'
-        '`cfy profiles set --manager-token <token>`\n'
-        '`cfy profiles set --kerberos-env <kerberos env>`\n'
-        '`cfy profiles set --manager-username <username> '
-        '--manager-password <password>`\n'
-    )
-
-    check_configured_auth(credentials, token, kerberos_env,
-                          extra_help=configure_help)
-
 
 def is_manager_active():
     active_profile = get_active_profile()
@@ -180,68 +158,26 @@ def is_manager_active():
 
 
 def get_profile_context(profile_name=None, suppress_error=False):
-    if profile_name == 'local':
-        return ProfileContext({}, profile_name='local')
-
+    # empty profile with nothing but default values
+    default = ProfileContext()
     profile_name = profile_name or get_active_profile()
-    loaded = None
-    path = get_context_path(profile_name)
-    if path:
-        try:
-            with open(path) as f:
-                loaded = json.load(f)
-        except IOError as e:
-            if e.errno != errno.ENOENT:
-                raise
-
-    if not loaded:
-        loaded = _try_load_yaml_profile(profile_name)
-
-    if not loaded:
+    if profile_name == 'local':
         if suppress_error:
-            return ProfileContext({})
-        raise CloudifyCliError('No context for profile {0}.\nPlease define '
-                               'the profile using `cfy profiles use`'
-                               .format(profile_name))
-    return ProfileContext(loaded, profile_name)
-
-
-class _ProfileLoader(yaml.SafeLoader):
-    """A yaml Loader that can load Cloudify 5.1 profiles
-
-    It supports python/unicode, which was commonly present in py2-created
-    profiles.
-    """
-
-
-def _load_str(loader, node):
-    return node.value
-
-
-_ProfileLoader.add_constructor(u'tag:yaml.org,2002:python/unicode', _load_str)
-
-
-def _try_load_yaml_profile(profile_name):
-    """Try to load the profile from the yaml context file.
-
-    This keeps compatibility with Cloudify 5.1 and earlier, who stored
-    the context in a yaml file. We will still load them, but we won't
-    store them anymore.
-    """
-    base_dir = get_profile_dir(profile_name)
-    if not base_dir:
-        return
+            return default
+        raise CloudifyCliError('Local profile does not have context')
     try:
-        with open(os.path.join(base_dir, 'context')) as f:
-            # dropping the object tag from yaml, so that we load the
-            # yaml as just a dict and not as an object
-            data = f.read().replace('!CloudifyProfileContext', '')
-            context = yaml.load(data, Loader=_ProfileLoader)
-    except IOError as e:
-        if e.errno != errno.ENOENT:
+        path = get_context_path(profile_name)
+        with open(path) as f:
+            context = yaml.load(f.read())
+        # fill the default with values from existing profile (the default is
+        # used as base because some of the attributes may not be in the
+        # existing profile file)
+        for key, value in context.__dict__.items():
+            setattr(default, key, value)
+    except CloudifyCliError:
+        if not suppress_error:
             raise
-        return
-    return context
+    return default
 
 
 def config_initialized_with_logging():
@@ -270,18 +206,25 @@ def is_initialized(profile_name=None):
         return config_initialized_with_logging()
 
 
-def get_context_path(profile_name):
-    base_dir = get_profile_dir(profile_name)
+def get_context_path(profile_name, suppress_error=False):
+    base_dir = get_profile_dir(profile_name, suppress_error)
     if not base_dir:
         return
-    return os.path.join(base_dir, 'context.json')
+    return os.path.join(
+        base_dir,
+        constants.CLOUDIFY_PROFILE_CONTEXT_FILE_NAME
+    )
 
 
-def get_profile_dir(profile_name=None):
+def get_profile_dir(profile_name=None, suppress_error=False):
     active_profile = profile_name or get_active_profile()
     if active_profile and os.path.isdir(
             os.path.join(PROFILES_DIR, active_profile)):
         return os.path.join(PROFILES_DIR, active_profile)
+    elif suppress_error:
+        return
+    else:
+        raise CloudifyCliError('Profile directory does not exist')
 
 
 def raise_uninitialized():
@@ -310,42 +253,47 @@ def get_rest_client(client_profile=None,
                     tenant_name=None,
                     trust_all=False,
                     cluster=None,
-                    kerberos_env=None,
-                    token=None):
+                    kerberos_env=None):
     if client_profile is None:
         client_profile = profile
-    assert_credentials_set(client_profile)
+    rest_host = rest_host or client_profile.manager_ip
+    rest_port = rest_port or client_profile.rest_port
+    rest_protocol = rest_protocol or client_profile.rest_protocol
+    rest_cert = rest_cert or get_ssl_cert(client_profile)
     username = username or get_username(client_profile)
     password = password or get_password(client_profile)
-    token = token or get_token(client_profile)
     tenant_name = tenant_name or get_tenant_name(client_profile)
+    trust_all = trust_all or get_ssl_trust_all()
+    headers = get_auth_header(username, password)
+    headers[constants.CLOUDIFY_TENANT_HEADER] = tenant_name
     cluster = cluster or is_cluster(client_profile)
     kerberos_env = kerberos_env \
         if kerberos_env is not None else client_profile.kerberos_env
 
-    kwargs = {
-        'host': rest_host or client_profile.manager_ip,
-        'port': rest_port or client_profile.rest_port,
-        'protocol': rest_protocol or client_profile.rest_protocol,
-        'cert': rest_cert or get_ssl_cert(client_profile),
-        'headers': {constants.CLOUDIFY_TENANT_HEADER: tenant_name},
-        'trust_all': trust_all or get_ssl_trust_all(),
-    }
-
-    if token:
-        kwargs['token'] = token
-    elif kerberos_env:
-        kwargs['kerberos_env'] = kerberos_env
-    else:
-        kwargs['username'] = username
-        kwargs['password'] = password
-        kwargs['headers'].update(get_auth_header(username, password))
+    if kerberos_env is False \
+            or (kerberos_env is None and not is_kerberos_env()):
+        if not username:
+            raise CloudifyCliError('Command failed: Missing Username')
+        if not password:
+            raise CloudifyCliError('Command failed: Missing password')
 
     if cluster:
-        kwargs['profile'] = client_profile
-        client = CloudifyClusterClient(**kwargs)
+        client = CloudifyClusterClient(host=rest_host,
+                                       port=rest_port,
+                                       protocol=rest_protocol,
+                                       headers=headers,
+                                       cert=rest_cert,
+                                       trust_all=trust_all,
+                                       profile=client_profile,
+                                       kerberos_env=kerberos_env)
     else:
-        client = CloudifyClient(**kwargs)
+        client = CloudifyClient(host=rest_host,
+                                port=rest_port,
+                                protocol=rest_protocol,
+                                headers=headers,
+                                cert=rest_cert,
+                                trust_all=trust_all,
+                                kerberos_env=kerberos_env)
     return client
 
 
@@ -364,56 +312,53 @@ def build_host_string(ip, ssh_user=''):
 
 
 def get_default_rest_cert_local_path():
-    base_dir = get_profile_dir() or CLOUDIFY_WORKDIR
+    base_dir = get_profile_dir(suppress_error=True) or CLOUDIFY_WORKDIR
     return os.path.join(base_dir, constants.PUBLIC_REST_CERT)
 
 
-def get_from_profile_or_env_var(key, from_profile=None):
+def get_username(from_profile=None):
     if from_profile is None:
         from_profile = profile
-
-    env_var = 'CLOUDIFY_{}'.format(key.upper())
-    profile_key = key
-    if key != 'kerberos_env':
-        profile_key = 'manager_' + key
-
-    env_value = os.environ.get(env_var)
-    profile_value = getattr(from_profile, profile_key)
-    if env_value and profile_value:
-        raise CloudifyCliError(
-            'Manager {title} is set in profile *and* in the `{env_var}` env '
-            'variable. Resolve the conflict before continuing.\n'
-            'Either unset the env variable, or run `cfy profiles unset '
-            '--manager-{key}'.format(
-                title=key.title(),
-                env_var=env_var,
-                key=key,
-            )
-        )
-    return env_value or profile_value
-
-
-def get_username(from_profile=None):
-    return get_from_profile_or_env_var('username', from_profile)
+    username = os.environ.get(constants.CLOUDIFY_USERNAME_ENV)
+    if username and from_profile.manager_username:
+        raise CloudifyCliError('Manager Username is set in profile *and* in '
+                               'the `CLOUDIFY_USERNAME` env variable. Resolve '
+                               'the conflict before continuing.\n'
+                               'Either unset the env variable, or run '
+                               '`cfy profiles unset --manager-username`')
+    return username or from_profile.manager_username
 
 
 def get_password(from_profile=None):
-    return get_from_profile_or_env_var('password', from_profile)
-
-
-def get_token(from_profile=None):
-    return get_from_profile_or_env_var('token', from_profile)
-
-
-def get_kerberos_env(from_profile=None):
-    return get_from_profile_or_env_var('kerberos_env', from_profile)
+    if from_profile is None:
+        from_profile = profile
+    password = os.environ.get(constants.CLOUDIFY_PASSWORD_ENV)
+    if password and from_profile.manager_password:
+        raise CloudifyCliError('Manager Password is set in profile *and* in '
+                               'the `CLOUDIFY_PASSWORD` env variable. Resolve '
+                               'the conflict before continuing.\n'
+                               'Either unset the env variable, or run '
+                               '`cfy profiles unset --manager-password`')
+    return password or from_profile.manager_password
 
 
 def get_tenant_name(from_profile=None):
-    return (
-        get_from_profile_or_env_var('tenant', from_profile)
-        or 'default_tenant'
-    )
+    if from_profile is None:
+        from_profile = profile
+    tenant = os.environ.get(constants.CLOUDIFY_TENANT_ENV)
+    if tenant and from_profile.manager_tenant:
+        raise CloudifyCliError('Manager Tenant is set in profile *and* in '
+                               'the `CLOUDIFY_TENANT` env variable. Resolve '
+                               'the conflict before continuing.\n'
+                               'Either unset the env variable, or run '
+                               '`cfy profiles unset --manager-tenant`')
+    return tenant or from_profile.manager_tenant
+
+
+def get_kerberos_env(from_profile=None):
+    if from_profile is None:
+        from_profile = profile
+    return from_profile.kerberos_env
 
 
 def get_ssl_cert(from_profile=None):
@@ -448,6 +393,11 @@ def get_ssl_trust_all():
     return False
 
 
+def get_version_data():
+    data = pkgutil.get_data('cloudify_cli', 'VERSION')
+    return json.loads(data)
+
+
 def get_manager_version_data(rest_client=None):
     if not rest_client:
         if not get_profile_context(suppress_error=True):
@@ -456,54 +406,92 @@ def get_manager_version_data(rest_client=None):
             rest_client = get_rest_client()
         except CloudifyCliError:
             return None
-    version_data = rest_client.manager.get_version()
+    try:
+        version_data = rest_client.manager.get_version()
+    except CloudifyClientError:
+        return None
     version_data['ip'] = profile.manager_ip
     return version_data
 
 
-class ProfileContext(object):
-    def __init__(self, context=None, profile_name=None):
-        self._context = {
-            'name': profile_name,
-            'manager_ip': None,
-            'ssh_key': None,
-            'ssh_port': 22,
-            'ssh_user': None,
-            'provider_context': {},
-            'manager_username': None,
-            'manager_password': None,
-            'manager_token': None,
-            'manager_tenant': None,
-            'rest_port': constants.DEFAULT_REST_PORT,
-            'rest_protocol': constants.DEFAULT_REST_PROTOCOL,
-            'rest_certificate': None,
-            'kerberos_env': False,
-            'cluster': {},
-        }
-        if context:
-            self._context.update(context)
+class ProfileContext(yaml.YAMLObject):
+    yaml_tag = u'!CloudifyProfileContext'
+    yaml_loader = yaml.Loader
 
-    def __getattr__(self, name):
-        return self._context[name]
-
-    def __setattr__(self, name, value):
-        if name in ['_context', 'profile_name']:
-            super(ProfileContext, self).__setattr__(name, value)
-        else:
-            self._context[name] = value
+    def __init__(self, profile_name=None):
+        # Note that __init__ is not called when loading from yaml.
+        # When adding a new ProfileContext attribute, make sure that
+        # all methods handle the case when the attribute is missing
+        self._profile_name = profile_name
+        self.manager_ip = None
+        self.ssh_key = None
+        self._ssh_port = None
+        self.ssh_user = None
+        self.provider_context = dict()
+        self.manager_username = None
+        self.manager_password = None
+        self.manager_tenant = None
+        self.rest_port = constants.DEFAULT_REST_PORT
+        self.rest_protocol = constants.DEFAULT_REST_PROTOCOL
+        self.rest_certificate = None
+        self.kerberos_env = False
+        self._cluster = dict()
 
     def to_dict(self):
-        ctx = self._context.copy()
-        ctx['name'] = self.profile_name
-        return ctx
+        return dict(
+            name=self.profile_name,
+            manager_ip=self.manager_ip,
+            ssh_key_path=self.ssh_key,
+            ssh_port=self.ssh_port,
+            ssh_user=self.ssh_user,
+            provider_context=self.provider_context,
+            manager_username=self.manager_username,
+            manager_tenant=self.manager_tenant,
+            rest_port=self.rest_port,
+            rest_protocol=self.rest_protocol,
+            rest_certificate=self.rest_certificate,
+            kerberos_env=self.kerberos_env,
+            cluster=self.cluster
+        )
+
+    @property
+    def ssh_port(self):
+        return self._ssh_port
+
+    @ssh_port.setter
+    def ssh_port(self, ssh_port):
+        # If the port is int, we want to change it to a string. Otherwise,
+        # leave None as is
+        ssh_port = str(ssh_port) if ssh_port else None
+        self._ssh_port = ssh_port
 
     @property
     def profile_name(self):
-        return self._context['name'] or self._context['manager_ip']
+        return getattr(self, '_profile_name', None) \
+            or getattr(self, 'manager_ip', None)
+
+    @property
+    def cluster(self):
+        # default the ._cluster attribute here, so that all callers can use it
+        # as just ._cluster, even if it's not present in the source yaml
+        if not hasattr(self, '_cluster'):
+            self._cluster = dict()
+        return self._cluster
+
+    @cluster.setter
+    def cluster(self, cluster):
+        self._cluster = cluster
 
     @profile_name.setter
-    def profile_name(self, value):
-        self._context['name'] = value
+    def profile_name(self, profile_name):
+        self._profile_name = profile_name
+
+    def _get_context_path(self):
+        init_path = get_profile_dir(self.profile_name)
+        context_path = os.path.join(
+            init_path,
+            constants.CLOUDIFY_PROFILE_CONTEXT_FILE_NAME)
+        return context_path
 
     @property
     def workdir(self):
@@ -516,22 +504,21 @@ class ProfileContext(object):
         workdir = destination or self.workdir
         # Create a new file
         if not os.path.exists(workdir):
-            os.makedirs(workdir, mode=0o700)
+            os.makedirs(workdir)
         target_file_path = os.path.join(
             workdir,
-            'context.json')
+            constants.CLOUDIFY_PROFILE_CONTEXT_FILE_NAME)
+
         with open(target_file_path, 'w') as f:
-            json.dump(self.to_dict(), f, sort_keys=True, indent=4)
-            f.write('\n')
+            f.write(yaml.dump(self))
 
 
 def get_auth_header(username, password):
     header = {}
 
     if username and password:
-        # encode/decode just to allow b64encode, which requires bytes
-        credentials = '{0}:{1}'.format(username, password).encode('utf-8')
-        encoded_credentials = b64encode(credentials).decode('utf-8')
+        credentials = '{0}:{1}'.format(username, password)
+        encoded_credentials = urlsafe_b64encode(credentials)
         header = {
             constants.CLOUDIFY_AUTHENTICATION_HEADER:
                 constants.BASIC_AUTH_PREFIX + ' ' + encoded_credentials}
@@ -550,7 +537,6 @@ def get_auth_header(username, password):
 # the same for every node in the cluster.
 CLUSTER_NODE_ATTRS = ['host_ip', 'host_type', 'rest_port', 'rest_protocol',
                       'ssh_port', 'ssh_user', 'ssh_key']
-_TRY_NEXT_NODE = object()
 
 
 class ClusterHTTPClient(HTTPClient):
@@ -583,14 +569,14 @@ class ClusterHTTPClient(HTTPClient):
 
         manager_host = get_target_manager()
         if manager_host:
-            self.host = ipv6_url_compat(manager_host)
-            return super(ClusterHTTPClient, self).do_request(*args, **kwargs)
+            self.host = manager_host
+            return self._try_do_request(*args, **kwargs)
 
         # First try with the main manager ip given when creating the profile
         # with `cfy profiles use`
-        self.host = ipv6_url_compat(self._profile.manager_ip)
+        self.host = self._profile.manager_ip
         response = self._try_do_request(*args, **kwargs)
-        if response is not _TRY_NEXT_NODE:
+        if response:
             return response
 
         for node_index, node in list(enumerate(
@@ -602,9 +588,8 @@ class ClusterHTTPClient(HTTPClient):
                 kwargs['data'] = copied_data[node_index]
 
             response = self._try_do_request(*args, **kwargs)
-            if response is _TRY_NEXT_NODE:
-                continue
-            return response
+            if response:
+                return response
 
         raise CloudifyClientError('All cluster nodes are offline')
 
@@ -616,15 +601,11 @@ class ClusterHTTPClient(HTTPClient):
                 CloudifyClientError) as e:
             if isinstance(e, CloudifyClientError) and e.status_code != 502:
                 raise
-            self.logger.warning('Could not connect to manager %s on port %s',
-                                self.host, self.port)
-            self.logger.debug(str(e))
-        return _TRY_NEXT_NODE
 
     def _use_node(self, node):
-        if ipv6_url_compat(node['host_ip']) == self.host:
+        if node['host_ip'] == self.host:
             return
-        self.host = ipv6_url_compat(node['host_ip'])
+        self.host = node['host_ip']
         for attr in ['rest_port', 'rest_protocol', 'trust_all', 'cert']:
             new_value = node.get(attr)
             if new_value:
@@ -641,7 +622,7 @@ class ClusterHTTPClient(HTTPClient):
         """
         self._profile.cluster[CloudifyNodeType.MANAGER].remove(node)
         self._profile.cluster[CloudifyNodeType.MANAGER] = (
-            [node] + self._profile.cluster[CloudifyNodeType.MANAGER])
+                [node] + self._profile.cluster[CloudifyNodeType.MANAGER])
         for node_attr in CLUSTER_NODE_ATTRS:
             if node_attr in node:
                 setattr(self._profile, node_attr, node[node_attr])
