@@ -19,12 +19,11 @@ import time
 
 from cloudify_rest_client.executions import Execution
 
-from cloudify_cli.exceptions import (
-    ExecutionTimeoutError,
-    EventProcessingTimeoutError)
+from .exceptions import (ExecutionTimeoutError,
+                         EventProcessingTimeoutError)
 
 
-WAIT_FOR_EXECUTION_SLEEP_INTERVAL = 1
+WAIT_FOR_EXECUTION_SLEEP_INTERVAL = 3
 WORKFLOW_END_TYPES = {u'workflow_succeeded', u'workflow_failed',
                       u'workflow_cancelled'}
 
@@ -41,19 +40,17 @@ class ExecutionEventsFetcher(object):
 
     def __init__(self,
                  client,
+                 execution_id,
                  batch_size=100,
-                 **list_kwargs):
+                 include_logs=False):
         self._client = client
-        self._list_kwargs = list_kwargs
+        self._execution_id = execution_id
         self._batch_size = batch_size
         self._from_event = 0
-        # make sure execution/group exists before proceeding
+        self._include_logs = include_logs
+        # make sure execution exists before proceeding
         # a 404 will be raised otherwise
-        if 'execution_id' in list_kwargs:
-            self._client.executions.get(list_kwargs['execution_id'])
-        elif 'execution_group_id' in list_kwargs:
-            self._client.execution_groups.get(
-                list_kwargs['execution_group_id'])
+        self._client.executions.get(execution_id)
 
     def fetch_and_process_events_batch(self,
                                        events_handler=None,
@@ -74,18 +71,18 @@ class ExecutionEventsFetcher(object):
         offset = offset if offset is not None else self._from_event
         size = size if size is not None else self._batch_size
         events_list_response = self._client.events.list(
+            execution_id=self._execution_id,
             _offset=offset,
             _size=size,
-            sort='reported_timestamp',
-            **self._list_kwargs
-        )
+            include_logs=self._include_logs,
+            sort='reported_timestamp')
         self._from_event += len(events_list_response)
         return events_list_response
 
     def _map_api_event_to_internal_event(self, event):
         """Map data structure from API to internal.
 
-        This method adapts the data structure returned by the events API
+        This method adapts the data structure returend by the events API
         endpoint to the structure expected by `cloudify.event.Event`.
 
         Note: the event is modified in place, so even though the value is
@@ -122,6 +119,7 @@ class ExecutionEventsFetcher(object):
 
     def fetch_and_process_events(self, events_handler=None, timeout=60):
         total_events_count = 0
+
         # timeout can be None (never time out), for example when tail is used
         if timeout is not None:
             deadline = time.time() + timeout
@@ -129,8 +127,7 @@ class ExecutionEventsFetcher(object):
         while True:
             if timeout is not None and time.time() > deadline:
                 raise EventProcessingTimeoutError(
-                    self._list_kwargs.get('execution_id') or
-                    self._list_kwargs.get('execution_group_id'),
+                    self._execution_id,
                     'events/log fetching timed out')
 
             events_batch_count, _ = self.fetch_and_process_events_batch(
@@ -155,13 +152,40 @@ def get_deployment_environment_creation_execution(client, deployment_id):
         'execution. Available executions: {0}'.format(executions))
 
 
+class EventsWatcher(object):
+    """Wraps an event_handler function, examines events to check if an
+    workflow execution finished has arrived.
+
+    This will set its .end_log_received instance attribute to True,
+    when it receives an event of type workflow_succeeded, workflow_cancelled
+    or workflow_failed.
+
+    :ivar end_log_received: was a "workflow execution finished" event seen?
+    :vartype end_log_received: bool
+    """
+
+    def __init__(self, events_handler=None):
+        self._events_handler = events_handler
+        self.end_log_received = False
+
+    def __call__(self, events):
+        if self._events_handler is not None:
+            self._events_handler(events)
+
+        if any(self._is_end_event(evt) for evt in events):
+            self.end_log_received = True
+
+    def _is_end_event(self, event):
+        """Is event a 'workflow execution finished' event?"""
+        return event.get('event_type') in WORKFLOW_END_TYPES
+
+
 def wait_for_execution(client,
                        execution,
                        events_handler=None,
                        include_logs=False,
                        timeout=900,
-                       logger=None,
-                       from_datetime=None):
+                       logger=None):
 
     # if execution already ended - return without waiting
     if execution.status in Execution.END_STATES:
@@ -171,12 +195,17 @@ def wait_for_execution(client,
         deadline = time.time() + timeout
 
     events_fetcher = ExecutionEventsFetcher(client,
-                                            execution_id=execution.id,
-                                            include_logs=include_logs,
-                                            from_datetime=from_datetime)
+                                            execution.id,
+                                            include_logs=include_logs)
 
     # Poll for execution status and execution logs, until execution ends
+    # and we receive an event of type in WORKFLOW_END_TYPES
     execution_ended = False
+    events_watcher = EventsWatcher(events_handler)
+
+    # did we already see the execution status change, and are only waiting
+    # for additional logs now?
+    waiting_for_logs = False
 
     while True:
         if timeout is not None:
@@ -194,66 +223,29 @@ def wait_for_execution(client,
             execution = client.executions.get(execution.id)
             execution_ended = execution.status in Execution.END_STATES
 
-        events_fetcher.fetch_and_process_events(
-            events_handler=events_handler, timeout=timeout)
+        if not events_watcher.end_log_received and \
+                execution.status != Execution.PENDING:
+            events_fetcher.fetch_and_process_events(
+                events_handler=events_watcher, timeout=timeout)
 
-        if execution_ended:
+        if execution_ended and events_watcher.end_log_received:
             break
 
+        # if the execution ended, wait one iteration for additional logs
+        if execution_ended:
+            if waiting_for_logs:
+                if logger:
+                    logger.info('Execution ended, but no end log message '
+                                'received. Some logs might not have been '
+                                'displayed.')
+                break
+            else:
+                if logger:
+                    logger.info('Execution ended, waiting {0} seconds for '
+                                'additional log messages'
+                                .format(WAIT_FOR_EXECUTION_SLEEP_INTERVAL))
+                waiting_for_logs = True
+
         time.sleep(WAIT_FOR_EXECUTION_SLEEP_INTERVAL)
 
     return execution
-
-
-def wait_for_execution_group(client,
-                             execution_group,
-                             events_handler=None,
-                             include_logs=False,
-                             timeout=900,
-                             logger=None,
-                             from_datetime=None):
-    """Like wait_for_execution, but for a group"""
-    if execution_group.status in Execution.END_STATES:
-        return execution_group
-
-    if logger is not None and execution_group.status == Execution.QUEUED:
-        logger.info("Executions have been queued: you can keep waiting "
-                    "for the executions to start or interrupt (eg. ^C).\n")
-
-    if timeout is not None:
-        deadline = time.time() + timeout
-
-    events_fetcher = ExecutionEventsFetcher(
-        client,
-        execution_group_id=execution_group.id,
-        include_logs=include_logs,
-        from_datetime=from_datetime)
-
-    # Poll for execution status and execution logs, until execution ends
-    group_finished = False
-
-    while True:
-        if timeout is not None:
-            if time.time() > deadline:
-                raise ExecutionTimeoutError(
-                    execution_group.id,
-                    'execution of operation {0} for deployment group {1} '
-                    'timed out'.format(execution_group.workflow_id,
-                                       execution_group.deployment_group_id))
-            else:
-                # update the remaining timeout
-                timeout = deadline - time.time()
-
-        if not group_finished:
-            execution_group = client.execution_groups.get(execution_group.id)
-            group_finished = execution_group.status in Execution.END_STATES
-
-        events_fetcher.fetch_and_process_events(
-            events_handler=events_handler, timeout=timeout)
-
-        if group_finished:
-            break
-
-        time.sleep(WAIT_FOR_EXECUTION_SLEEP_INTERVAL)
-
-    return execution_group
