@@ -17,54 +17,40 @@
 import os
 import sys
 import json
-import time
 import click
 import errno
 import string
 import random
 import shutil
-import logging
 import tarfile
 import zipfile
 import tempfile
+import collections
 from shutil import copy
+from urlparse import urlparse
 from contextlib import closing, contextmanager
 from backports.shutil_get_terminal_size import get_terminal_size
 
-try:
-    from collections.abc import MutableMapping
-except ImportError:
-    from collections import MutableMapping
-
-import yaml
 import requests
-from retrying import retry
-from urllib.parse import urlparse
 
-from cloudify_cli.constants import SUPPORTED_ARCHIVE_TYPES, DEFAULT_TIMEOUT
-from cloudify_cli.exceptions import CloudifyCliError, CloudifyTimeoutError
-from cloudify_cli.execution_events_fetcher import ExecutionEventsFetcher
-from cloudify_cli.logger import get_logger, get_events_logger
+from .logger import get_logger
+from .exceptions import CloudifyCliError
+from .constants import SUPPORTED_ARCHIVE_TYPES
 
-from cloudify.models_states import BlueprintUploadState
 from cloudify_rest_client.constants import VisibilityState
 from cloudify_rest_client.exceptions import CloudifyClientError
 
-WAIT_FOR_BLUEPRINT_UPLOAD_SLEEP_INTERVAL = 1
-
 
 def get_deployment_environment_execution(client, deployment_id, workflow):
-    executions = client.executions.list(deployment_id=deployment_id,
-                                        workflow_id=workflow,
-                                        sort='created_at',
-                                        is_descending=True)
 
-    if executions and len(executions) > 0:
-        return executions[0]
+    executions = client.executions.list(deployment_id=deployment_id)
+    for execution in executions:
+        if execution.workflow_id == workflow:
+            return execution
 
     raise RuntimeError(
-        'Failed to get {0} workflow execution for deployment {1}'.format(
-            workflow, deployment_id)
+        'Failed to get {0} workflow execution.'
+        ' Available executions: {1}'.format(workflow, executions)
     )
 
 
@@ -75,9 +61,6 @@ def dump_to_file(collection, file_path):
 
 
 def is_virtual_env():
-    if hasattr(sys, 'base_prefix'):
-        # py3 case, with the stdlib venv
-        return sys.base_prefix != sys.prefix
     return hasattr(sys, 'real_prefix')
 
 
@@ -88,6 +71,34 @@ def get_cwd():
     return os.getcwd()
 
 
+def decode_list(data):
+    rv = []
+    for item in data:
+        if isinstance(item, unicode):
+            item = item.encode('utf-8')
+        elif isinstance(item, list):
+            item = decode_list(item)
+        elif isinstance(item, dict):
+            item = decode_dict(item)
+        rv.append(item)
+    return rv
+
+
+def decode_dict(data):
+    rv = {}
+    for key, value in data.iteritems():
+        if isinstance(key, unicode):
+            key = key.encode('utf-8')
+        if isinstance(value, unicode):
+            value = value.encode('utf-8')
+        elif isinstance(value, list):
+            value = decode_list(value)
+        elif isinstance(value, dict):
+            value = decode_dict(value)
+        rv[key] = value
+    return rv
+
+
 def remove_if_exists(path):
 
     try:
@@ -149,12 +160,12 @@ def zip_files(files):
     destination_zip = source_folder + '.zip'
     for path in files:
         copy(path, source_folder)
-    create_zip(source_folder, destination_zip, include_folder=False)
+    zip(source_folder, destination_zip, include_folder=False)
     shutil.rmtree(source_folder)
     return destination_zip
 
 
-def create_zip(source, destination, include_folder=True):
+def zip(source, destination, include_folder=True):
     logger = get_logger()
 
     logger.debug('Creating zip archive: {0}...'.format(destination))
@@ -245,8 +256,8 @@ def generate_progress_handler(file_path, action='', max_bar_length=80):
 
     # Shorten the file name if it's too long
     file_name = os.path.basename(file_path)
-    if len(file_name) > (bar_length // 4) + 3:
-        file_name = file_name[:bar_length // 4] + '...'
+    if len(file_name) > (bar_length / 4) + 3:
+        file_name = file_name[:bar_length / 4] + '...'
 
     bar_length -= len(file_name)
 
@@ -281,7 +292,7 @@ def handle_client_error(status_code, message, logger):
     """
     try:
         yield
-    except CloudifyClientError as e:
+    except CloudifyClientError, e:
         if e.status_code != status_code:
             raise
         logger.info(message)
@@ -296,10 +307,10 @@ def prettify_client_error(status_codes, logger):
     """
     try:
         yield
-    except CloudifyClientError as e:
+    except CloudifyClientError, e:
         if e.status_code not in status_codes:
             raise
-        logger.error('Error: %s', e)
+        logger.info(e.message)
 
 
 def get_visibility(private_resource,
@@ -350,9 +361,9 @@ def explicit_tenant_name_message(tenant_name, logger):
 
 
 def deep_update_dict(dest_dict, src_dict):
-    for key, value in src_dict.items():
-        if isinstance(dest_dict, MutableMapping):
-            if isinstance(value, MutableMapping):
+    for key, value in src_dict.iteritems():
+        if isinstance(dest_dict, collections.MutableMapping):
+            if isinstance(value, collections.MutableMapping):
                 dest_dict[key] = deep_update_dict(dest_dict.get(key), value)
             else:
                 dest_dict[key] = src_dict[key]
@@ -362,8 +373,8 @@ def deep_update_dict(dest_dict, src_dict):
 
 
 def deep_subtract_dict(dest_dict, src_dict):
-    for key, value in src_dict.items():
-        if isinstance(value, MutableMapping):
+    for key, value in src_dict.iteritems():
+        if isinstance(value, collections.MutableMapping):
             deep_subtract_dict(dest_dict.get(key), value)
         else:
             if key not in dest_dict:
@@ -394,8 +405,8 @@ def assert_one_argument(arguments):
     """
     filtered = [k for k in arguments if arguments[k]]
     if len(filtered) != 1:
-        raise CloudifyCliError('Please provide one of the options: ' +
-                               ', '.join(arguments))
+        raise CloudifyCliError('Please provide one of the options: '
+                               '{0}'.format(arguments.keys()))
 
 
 def load_json(input_path):
@@ -410,76 +421,3 @@ def print_dict(keys_dict, logger):
     for key, values in keys_dict.items():
         str_values = [str(value) for value in values]
         logger.info('{0}: {1}'. format(key, str_values))
-
-
-def get_dict_from_yaml(yaml_path):
-    with open(yaml_path) as f:
-        return yaml.load(f, yaml.Loader)
-
-
-def wait_for_blueprint_upload(client, blueprint_id, logging_level):
-
-    def _handle_errors():
-        if blueprint['state'] in BlueprintUploadState.FAILED_STATES:
-            error_msg = '{error_type} blueprint: {description}.'.format(
-                error_type=blueprint['state'].capitalize().replace('_', ' '),
-                description=blueprint['error']
-            )
-            if logging_level == logging.DEBUG:
-                error_msg += '\nError traceback: {}'.format(
-                    blueprint['error_traceback'])
-            raise CloudifyCliError(error_msg)
-
-    @retry(stop_max_attempt_number=5, wait_fixed=1000)
-    def _get_blueprint_and_upload_execution_id():
-        bp = client.blueprints.get(blueprint_id)
-        # upload_execution['id'] might not be available at first, hence retry
-        return bp, bp.upload_execution['id']
-
-    try:
-        blueprint, execution_id = _get_blueprint_and_upload_execution_id()
-    except KeyError:
-        raise RuntimeError(
-            'Failed to get upload_blueprint workflow execution for blueprint '
-            '{0}.  That may indicate a problem with blueprint upload.  Verify '
-            'blueprint\'s state by running command `cfy blueprints get {0}`.'
-            .format(blueprint_id)
-        )
-
-    # if blueprint upload already ended - return without waiting
-    if blueprint['state'] in BlueprintUploadState.END_STATES:
-        _handle_errors()
-        return blueprint
-
-    deadline = time.time() + DEFAULT_TIMEOUT
-
-    events_fetcher = ExecutionEventsFetcher(
-        client, execution_id=execution_id, include_logs=True)
-
-    # Poll for execution status and execution logs, until execution ends
-    # and we receive an event of type in WORKFLOW_END_TYPES
-    upload_ended = False
-    events_handler = get_events_logger(None)
-
-    # Poll for blueprint upload status, until the upload ends
-    while True:
-        if time.time() > deadline:
-            raise CloudifyTimeoutError('Blueprint {0} upload timed '
-                                       'out'.format(blueprint.id))
-        timeout = deadline - time.time()  # update remaining timeout
-
-        if not upload_ended:
-            blueprint = client.blueprints.get(blueprint.id)
-            upload_ended = \
-                blueprint['state'] in BlueprintUploadState.END_STATES
-
-        events_fetcher.fetch_and_process_events(
-            events_handler=events_handler, timeout=timeout)
-
-        if upload_ended:
-            break
-
-        time.sleep(WAIT_FOR_BLUEPRINT_UPLOAD_SLEEP_INTERVAL)
-    blueprint = client.blueprints.get(blueprint_id)
-    _handle_errors()
-    return blueprint
